/**********************************************************************/
/*   By Stefan Pantos. E-Mail: Stefuser@yahoo.com                     */
/*	 Last Modified: 21/07/2000										  */
/**********************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
#include <utility.h>
#include <userint.h>
#include "Conversion.h"
#include "DiffConv.h"
#include "DiffConvTypes.h"
#include "Options.h"
#include "RangeOptPanel.h" 
#include "SplineOptionPanel.h"
#include "ConstantStepOptPanel.h"
#include "CPIOps.h" 

#define CHN -1// This is the number used to check that the opened file is a CHN file or not
#define kNUMOFPOINTSFORCONSSTEP 5

double startAngle;
double inputStep;

double round(double number)
{
	double tempNumber;
	
	tempNumber = (int)(number/1);
	if ((number - tempNumber)>=0.5)
	{
		tempNumber++;
	}
	return tempNumber;
}

void getDirPath(char dirPath[])
{
	int stringIndex=0; //initalize the string index
	
	while (dirPath[stringIndex]!=0x00) //While it's not pointing to the end of the file
	{
		stringIndex++; //Move to the next char in the string
	}
	while (dirPath[stringIndex]!='\\') //While stringIndex isn't pointing to a \ then
	{
		stringIndex--; //Go to the previose char
	}
	dirPath[stringIndex+1]=0x00;  //cut the file name of the end of the path.
}

void addExtension(char *string, char extension[3], int maxStringLen)
{
	int stringLen = strlen(string);
	int indexInExt = 0;
	
	string[stringLen]='.';
	stringLen++;
	while (indexInExt < 4 && extension[indexInExt]!=0x00 && stringLen < maxStringLen)
	{
		string[stringLen]=extension[indexInExt];
		indexInExt++;
		stringLen++;
	}
	string[stringLen]=0x00;
}

void findAFreeFileName(char *filePathWOExt, char extension[3], char *outputFileName, int maxLenOfOutputFilePath)
{
	char message[300];
	int outputFile;
	int fileUpTo = 0;
	int numOfZeros = 0;
	
	DisableBreakOnLibraryErrors();
	strcpy(outputFileName, filePathWOExt);
	addExtension(outputFileName, extension, maxLenOfOutputFilePath); //just add the extension .txt to the end of the file.
	if ((outputFile=OpenFile(outputFileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))>=00) //trys to open the file as read only to see if that file already exists.
	{
		sprintf(message, "Do you want to repalce\nthe file: %s?", outputFileName);  //Add the file name of the output file onto the other string and put the result into message
		Delay(0.1);
		if (!ConfirmPopup("Replace?", message))
		{
			do
			{
				fileUpTo++;
				strcpy(outputFileName, filePathWOExt);
				sprintf(outputFileName, "%s-", outputFileName);
				for (numOfZeros = 3-(int)(log(fileUpTo)); numOfZeros > 0; numOfZeros--)
				{
					sprintf(outputFileName, "%s0", outputFileName);
				}
				sprintf(outputFileName, "%s%d", outputFileName, fileUpTo);
				addExtension(outputFileName, extension, maxLenOfOutputFilePath); //just add the extension .txt to the end of the file.
			}
			while ((outputFile=OpenFile(outputFileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))>=0 && fileUpTo < 9999);
			if (outputFile>=0)
			{
				CloseFile(outputFile);
			}
		}
	}
	EnableBreakOnLibraryErrors();	     
}

void updateProgress(int channel, int max, int progressPanel)
{
	int progressValue;
	
	progressValue = (int)((float)((float)channel/(float)max)*100); //work out the amount of the file which has been processed.
	if ((progressValue)%10==0) //if the value is divisible by 10 then...
	{
		SetCtrlVal(progressPanel, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
	}
}

/************************removeExtension****************************/
/*  This removes the extension of a file name if it has one.       */
/* Parameters:													   */
/*      string: The path anem which is to have the extension       */
/*	removed from.	   											   */
/*******************************************************************/
void removeExtension(char *string)
{
	int indexInString = 0;
	indexInString = strlen(string);
	while (string[indexInString]!='.' && string[indexInString]!='\\' && string[indexInString]!=':' && indexInString>0)
	{
		indexInString--;
	}
	if (string[indexInString]=='.')
	{
		string[indexInString]=0x00;
	}
}

/*****************************convertToTTheta***********************/
/* This converts the channel number entered into the 2-theta	   */
/* This convert to 2-theta using either a linear relation ship of a*/
/* spline curve depending no the value of the linear convertion	   */
/* radio button.												   */
/*  Parameters:													   */
/*		channelNum: this is the value which is to be converted to  */
/*	   2-theta.													   */
/*	returns:													   */
/*		It returns the 2-theta value which corrasponds to the      */
/*     channel number past to it.								   */
/*******************************************************************/
double channelToTTheta(double channelNum, int linearValue)
{
	double tempValue; //Value for extending th spline curve.
	
	if (linearValue) //If the linear convertion radio button is on then...
	{
		return header.K+header.C*channelNum;  //Do the linear convertion.
	}
	else   //If the linear convertion radio button is off then...
	{
	 	if (calPoints[0].xPoint>channelNum || calPoints[header.numOfCalibPoints-1].xPoint<channelNum) //If the cahnnel number is out of the cal points range then...
	 	{
	 		tempValue = -(calPoints[header.numOfCalibPoints-1].yPoint-calPoints[0].yPoint)/(calPoints[header.numOfCalibPoints-1].xPoint-calPoints[0].xPoint);  //Work out the gradaint of the line between the first and last calibration points.
	 		return (calPoints[0].xPoint-channelNum)*tempValue+calPoints[0].yPoint;   //Uses a linear relationship to calculate the 2-theta value and return that value.
	 	}
	 	//else if the point is with in the range of the calibration points then...
		return splint(calPoints, header.numOfCalibPoints, channelNum);  //Uses spline to work out the 2-theta value.  
	}
	return 0;
}

double findCountForTThetaLin(calPoint *calPoints, long numOfChans, double tThetaVal)
{
	int start=0, end = numOfChans-1, index = (end-start)/2;
	double temp;
	
	while ((calPoints[index].xPoint != tThetaVal) && start!=end && start +1 != end)
	{
		if (calPoints[index].xPoint<tThetaVal)
		{
			start = index;	
		}
		else
		{
			end =index;	
		}
		index = (end-start)/2+start;
	}
	if (calPoints[index].xPoint == tThetaVal)
	{
		return calPoints[index].yPoint; 
	}
	else if (calPoints[index].xPoint > tThetaVal)
	{
		return (calPoints[index].yPoint-calPoints[index-1].yPoint)/(calPoints[index].xPoint-calPoints[index-1].xPoint)*(tThetaVal -calPoints[index-1].xPoint)+calPoints[index].yPoint;	
	}
	else
	{
		return (calPoints[index+1].yPoint-calPoints[index].yPoint)/(calPoints[index+1].xPoint-calPoints[index].xPoint)*(tThetaVal -calPoints[index].xPoint)+calPoints[index].yPoint;	
	}
	return -1000;
}

/**********************************getTThetaAndCounts************************/
/*  This is to get the 2-theta and the counts which coraspond to the channel*/
/* number of the 2-theta value past depending on whether it is a constant   */
/* step conversion.														    */
/*	Parameters:																*/
/*			tTheta: this is either a channel number or a 2-theta value which*/
/*    		needs a value to be returned for.								*/
/*			dataPoints: These are the points read in from the file.			*/
/*			header: This is a pointer to a header structure which relates to*/
/*			the current file which is being converted.						*/
/*			paramBlock: This is a structure of parameters which give 		*/
/*			information amout the current file.								*/
/*	Returns:																*/
/*			If there was an error eg the value was out side the rage limits */
/*			or a value couldn't be calculaited of the value past then it 	*/
/*			returns a non-zero value.										*/
/*			tTheta:	This returns the 2-theta value.							*/
/*			counts: This value if the number of counts which coraspond to 	*/
/*			the 2-theta value returned in tTheta.							*/
/****************************************************************************/
int getTThetaAndCounts(double *tTheta, double *counts, calPoint *dataPoints, void *header, convertParamBlock *paramBlock)
{
	int size;
	int currentChannel = 0;
	
	if (paramBlock->inputFileType == kCHN)
	{
		size = ((MCAFileHeader *)header)->num_chans; 
	}
	else
	{
		size = ((GUFIHeader*)header)->numPoints;
	}
	if ((kCONSTANTSTEP & paramBlock->conversionOtherOptions)!=0)
	{
		if (constStepOpts.conversionMethod == kSPLINE)
		{
			*counts = splint(dataPoints, size, *tTheta);
		}
		else
		{
			*counts = findCountForTThetaLin(dataPoints, size, *tTheta);
		}
	}
	else
	{
		if (size < (int)*tTheta)
		{
			return 1;
		}
		*counts = dataPoints[(int)*tTheta].yPoint;
		*tTheta = dataPoints[(int)*tTheta].xPoint;
	}
	if ((paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((*tTheta < theRangeOptions.startValue) || (*tTheta > theRangeOptions.endValue))
		{
			return -1;
		}
	}
	if ((paramBlock->conversionOtherOptions & kROUNDTOINT) != 0)
	{
		
		*tTheta = round(*tTheta);
	}
	if ((paramBlock->conversionOtherOptions & kFLOATINGCOUNTS) == 0)
	{
		*counts = round(*counts);
	}
	return 0;
}

/************************************GetHeaderInfo***************************/

void GetHeaderInfo(convertParamBlock *paramBlock, double step, double minTheta, long maxChannel, double *min, double *max, int numOfPoints)
{
	if (minTheta < theRangeOptions.startValue && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((((theRangeOptions.startValue-(minTheta))/(step))-(int)((theRangeOptions.startValue-(minTheta))/(step))) != 0)
		{
			*min = (int)(((theRangeOptions.startValue-(minTheta))/(step))+1)*(step)+(minTheta);
		}
		else
			*min = theRangeOptions.startValue;
	}
	else
	{
		*min = minTheta;
	}
	if (maxChannel*(step)+(minTheta) > theRangeOptions.endValue  && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((((theRangeOptions.endValue-(minTheta))/(step))-(int)((theRangeOptions.endValue-(minTheta))/(step))) != 0)
		{
			*max = ((int)((theRangeOptions.endValue-minTheta)/(step)))*(step)+(minTheta);
		}
		else
		{
			*max = theRangeOptions.endValue;
		}
	}
	else
	{
		*max = maxChannel*(step)+(minTheta);
	}
}

/*********************setUpForConstantStep***********************/
/*  This should be called if the output has to be in a constant */
/* step using a spline interpolation. If calculaits the y'' for */
/* the data points passed in    								*/
/* dataPoints if the constant step it to be calculaited as using*/
/* a spline curve.												*/
/* parameters:													*/
/*		dataPoints: This is a pointer to the list of data points*/
/*		read in from the file.									*/
/*		header: A pointer to a header for either a MCA or a GUFI*/
/*		file.													*/
/*		conversionParamBlock: This is the a set of conversion   */
/*		parameters.												*/
/****************************************************************/
void setUpForConstantStep(calPoint *dataPoints, void *header, convertParamBlock *paramBlock)
{
	long size;  										//This is to store the number of the points which have been read in.
	
	if (constStepOpts.conversionMethod==kSPLINE)  		//If the constent step method isn't linear then the don't bother doing this. 
	{	
		if (paramBlock->inputFileType == kCHN)          //If the file to be converted is a CHN file then...
		{
			size = ((MCAFileHeader*)header)->num_chans; //Get the number of points from the CHN header.
		}
		else  //Else if the file is a GUFI file then...
		{
			size = ((GUFIHeader*)header)->numPoints; //Get the number of points from the GUFI header.
		}			  
		spline(dataPoints, size, paramBlock->progressPanelHandle, 0);  //Calculait all the y'' for all the points.
	}
}

int readMCAFile(convertParamBlock *paramBlock, calPoint **data, MCAFileHeader *MCAHeader)
{
	char message[300];
	int inputFile;
	long numOfBytesRead;
	long count, channel;
	
	if ((inputFile=OpenFile(paramBlock->inputFileName, VAL_READ_ONLY, VAL_APPEND, VAL_BINARY))<0)
	{
		sprintf(message, "Couldn't open input file: %s\nFile may not exist.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return -1;
	}
	numOfBytesRead = ReadFile(inputFile, ((char*)(MCAHeader)), sizeof(MCAFileHeader)); //read in the file header for the input file
	if (MCAHeader->fileType!=CHN)	//Check that it's got the correct file type
	{
		sprintf(message, "The input file \"%s\" \nwasn't a CHN file.", paramBlock->inputFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	if (*data!=NULL)
	{
		free(*data);
		*data = NULL;
	}
	*data = malloc(sizeof(calPoint)*MCAHeader->num_chans);  //allocate enough memory to load the while file into memory
	for (channel = 0; channel < MCAHeader->num_chans && numOfBytesRead>0; channel ++)
	{
		numOfBytesRead = ReadFile(inputFile, (char *)&count, sizeof(long));
		if (numOfBytesRead > 0)
		{
			((calPoint*)*data)[channel].yPoint = count;
			((calPoint*)*data)[channel].xPoint = channelToTTheta(channel, (paramBlock->conversionType & kTWOTHETALINEAR)!=0);
		}
	}
	if (channel != MCAHeader->num_chans)
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", paramBlock->inputFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		CloseFile(inputFile);
		return -1;  //And leg it.
	}
	CloseFile(inputFile);
	startAngle = MCAHeader->chan_offset;
	inputStep = 1;
	return 0;
}

int readGUFIFile(convertParamBlock *paramBlock, calPoint **data, GUFIHeader *header)
{
	char message[300];
	int inputFile;
	long numOfBytesRead, bytes=1;
	short i;
	float counts;
	
	if ((inputFile=OpenFile(paramBlock->inputFileName, VAL_READ_ONLY, VAL_APPEND, VAL_BINARY))<0)
	{
		sprintf(message, "Couldn't open input file: %s\nFile may not exist.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return -1;									   
	}
	SetFilePtr(inputFile, 0x09, 0);
	numOfBytesRead = ReadFile(inputFile, (char*)&(header->startAng), sizeof(float));
	if (numOfBytesRead != sizeof(float))
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", paramBlock->inputFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	SetFilePtr(inputFile, 0x0D, 0);
	numOfBytesRead = ReadFile(inputFile, (char*)&(header->stepAng), sizeof(float));
	if (numOfBytesRead != sizeof(float))
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", paramBlock->inputFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	SetFilePtr(inputFile, 0x11, 0);
	numOfBytesRead = ReadFile(inputFile, (char*)&(header->numPoints), sizeof(short));
	if (*data!=NULL)
	{
		free(*data);
		*data = NULL;
	}
	SetFilePtr(inputFile, 0x283, 0);
	*data = malloc(sizeof(calPoint)*header->numPoints);
	numOfBytesRead = 0;
	for (i=0; i<header->numPoints && bytes > 0; i++)
	{
		numOfBytesRead += bytes = ReadFile(inputFile, (char *)&counts, sizeof(float));
		((calPoint*)*data)[i].yPoint = counts;
		if (paramBlock->conversionType != kNULLCONVERSION)
		{
			((calPoint*)*data)[i].xPoint = channelToTTheta(header->startAng + i*header->stepAng, (paramBlock->conversionType & kTWOTHETALINEAR)!=0);
		}
		else
		{
			((calPoint*)*data)[i].xPoint = header->startAng + i*header->stepAng;
		}
	}
	if (numOfBytesRead != sizeof(float)*header->numPoints)
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", paramBlock->inputFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	CloseFile(inputFile);
	startAngle =header->startAng;
	inputStep = header->stepAng;
	return 0;
}

void writeTextFileChannels(convertParamBlock *paramBlock, MCAFileHeader *MCAHeader, calPoint *dataPoints)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	int progressValue=0;
	char lineToOutput[300];
	
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName);
	findAFreeFileName(paramBlock->outputFileName, "txt", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full or you may not have write permission.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return;	
	}
	WriteLine(outputFile, "Channel\tCount", 13);
	for (channel =0; channel<MCAHeader->num_chans; channel++)
	{
		updateProgress(channel, MCAHeader->num_chans, paramBlock->progressPanelHandle);
		sprintf(lineToOutput, "%d	%8.00f", channel+MCAHeader->chan_offset, dataPoints[channel].yPoint);	//conver the channel number to 2-theta and construct a line with the 2-theta value and the intensity on it.
		WriteLine(outputFile, lineToOutput, strlen(lineToOutput));
	}
	CloseFile(outputFile);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
}

void writeTThetaLine(int outputFile, short options, double tTheta, double counts, long channel)
{
	char lineToOutput[300];
	
	if ((options & kROUNDTOINT) != 0)
	{
		sprintf(lineToOutput, "%d	", (int)tTheta);	//convert the channel number to 2-theta and construct a line with the 2-theta value and the intensity on it.
	}
	else
	{	
		sprintf(lineToOutput, "%9.08f	", tTheta);	//convert the channel number to 2-theta and construct a line with the 2-theta value and the intensity on it.
	}	
	if ((options & kFLOATINGCOUNTS) == 0)
	{
		sprintf(lineToOutput, "%s%d", lineToOutput, (int)counts);
	}
	else
	{
		sprintf(lineToOutput, "%s%8.04f", lineToOutput, counts); 	
	}
	WriteLine(outputFile, lineToOutput, strlen(lineToOutput));
}

void writeTextFileTTheta(convertParamBlock *paramBlock, calPoint *dataPoints, void* header)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	double tTheta;
	double counts;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	
	if (paramBlock->inputFileType == kCHN)
	{
		size = ((MCAFileHeader *)header)->num_chans; 
	}
	else
	{
		size = ((GUFIHeader*)header)->numPoints;
	}
	findAFreeFileName(paramBlock->outputFileName, "txt", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return;	
	}
	if ((paramBlock->conversionOtherOptions & kCONSTANTSTEP)!=0)
	{
		setUpForConstantStep(dataPoints, header, paramBlock);
		step = constStepOpts.stepValue;
		minTheta = dataPoints[0].xPoint;
		if (minTheta < theRangeOptions.startValue && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
		{
			minTheta = theRangeOptions.startValue;
		}
		maxChannel = (int)(dataPoints[size-1].xPoint-minTheta)/step;
	}
	else
	{
		step = 1;
		minTheta = 0;
		maxChannel = size;
	}
	WriteLine(outputFile, "2-Theta\tCount", 13);
	for (channel =0; channel<maxChannel; channel++)
	{
		updateProgress(channel, maxChannel, paramBlock->progressPanelHandle);
		tTheta = minTheta + step * channel;
		if (0 == getTThetaAndCounts(&tTheta, &counts, dataPoints, header, paramBlock))
		{
			writeTThetaLine(outputFile, paramBlock->conversionOtherOptions, tTheta, counts, channel);
		}
	}
	CloseFile(outputFile);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
}

void writeXFITHeader(double start, double step, double stopVal, int fileHandle)
{
	char lineToOutput[300];
	WriteLine(fileHandle, "\n\n\n", 3);
	sprintf(lineToOutput, "%8.3f%8.3f%8.3f", start, step, stopVal);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
}

void writeCPIHeader(double start, double step, double stopVal, int fileHandle)
{
	char lineToOutput[300];
	
	WriteLine(fileHandle, "SIETRONICS XRD SCAN", 19);
	sprintf(lineToOutput, "%0.08f", start);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput)); 
	sprintf(lineToOutput, "%0.08f", stopVal);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput)); 
	sprintf(lineToOutput, "%0.08f", step); 
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
	WriteLine(fileHandle, CPIOpt.anodeSymbol, 2);
	sprintf(lineToOutput, "%0.5f", CPIOpt.waveLength);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
	WriteLine(fileHandle, DateStr(), strlen(DateStr()));
	WriteLine(fileHandle, "1", 1);
	WriteLine(fileHandle, "MCAtoOTHER", 10);
	WriteLine(fileHandle, "SCANDATA", 8);
}

void writeLHMPHeader(double start, double step, double stopVal, int fileHandle)
{
	char lineToOutput[300];
	sprintf(lineToOutput, "%8.3f%8.3f%8.3f", start, step, stopVal);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
}

void writeXFitCount(int outputFile, short options, double counts, long channel)
{
	char lineToOutput[300];
	
	if ((options & kFLOATINGCOUNTS) == 0)
	{
		sprintf(lineToOutput, "%8d", (int)counts);	
	}
	else
	{
		sprintf(lineToOutput, "%8.02f", counts);
	}
	WriteFile(outputFile, lineToOutput, strlen(lineToOutput));
	if ((channel+1)%10 == 0)
	{
		WriteFile(outputFile, "\n", 1);
	}
}

void writeXFITFile(convertParamBlock *paramBlock, calPoint *dataPoints, void* header)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	double tTheta;
	double counts;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	double min, max;
	
	if (paramBlock->inputFileType == kCHN)
	{
		size = ((MCAFileHeader *)header)->num_chans; 
	}
	else
	{
		size = ((GUFIHeader*)header)->numPoints;
	}
	findAFreeFileName(paramBlock->outputFileName, "dat", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return;	
	}
	if ((paramBlock->conversionOtherOptions & kCONSTANTSTEP)!=0)
	{
		setUpForConstantStep(dataPoints, header, paramBlock);
		step = constStepOpts.stepValue;
		minTheta = dataPoints[0].xPoint;
		if (minTheta < theRangeOptions.startValue && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
		{
			minTheta = theRangeOptions.startValue;
		}
		maxChannel = (int)(dataPoints[size-1].xPoint - minTheta)/step;
		}
	else
	{
		step = 1;
		minTheta = 0;
		maxChannel = size;
	}
	GetHeaderInfo(paramBlock, step, minTheta, maxChannel, &min, &max, size);
	writeXFITHeader(min, step, max, outputFile);
	for (channel =0; channel<maxChannel; channel++)
	{
		updateProgress(channel, maxChannel, paramBlock->progressPanelHandle);
		tTheta = minTheta + step * channel;
		if (0 == getTThetaAndCounts(&tTheta, &counts, dataPoints, header, paramBlock))
		{	
			writeXFitCount(outputFile, paramBlock->conversionOtherOptions, counts, channel); 
		}
	}
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	CloseFile(outputFile);
}

void writeCPICount(int outputFile, short options, double counts)
{
	char lineToOutput[300];
	
	if ((options & kFLOATINGCOUNTS) == 0)
	{
		sprintf(lineToOutput, "%d", (int)counts);	
	}
	else
	{
		sprintf(lineToOutput, "%8.04f", counts);
	}
	WriteLine(outputFile, lineToOutput, strlen(lineToOutput));
}

void writeCPIFile(convertParamBlock *paramBlock, calPoint *dataPoints, void* header)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	char lineToOutput[300];
	double tTheta;
	double counts;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	double min, max;
	
	if (paramBlock->inputFileType == kCHN)
	{
		size = ((MCAFileHeader *)header)->num_chans; 
	}
	else
	{
		size = ((GUFIHeader*)header)->numPoints;
	}
	findAFreeFileName(paramBlock->outputFileName, "cpi", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return;	
	}
	if ((paramBlock->conversionOtherOptions & kCONSTANTSTEP)!=0)
	{
		setUpForConstantStep(dataPoints, header, paramBlock);
		step = constStepOpts.stepValue;
		minTheta = dataPoints[0].xPoint;
		if (minTheta < theRangeOptions.startValue && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
		{
			minTheta = theRangeOptions.startValue;
		}
		maxChannel = (int)(dataPoints[size-1].xPoint-minTheta)/step;
	}
	else
	{
		step = 1;
		minTheta = 0;
		maxChannel = size;
	}
	GetHeaderInfo(paramBlock, step, minTheta, maxChannel, &min, &max, size);
	writeCPIHeader(min, step, max, outputFile);
	for (channel =0; channel<maxChannel; channel++)
	{
		updateProgress(channel, maxChannel, paramBlock->progressPanelHandle);
		lineToOutput[0]=0x00;
		tTheta = minTheta + step * channel;
		if (0 == getTThetaAndCounts(&tTheta, &counts, dataPoints, header, paramBlock))
		{	
			writeCPICount(outputFile, paramBlock->conversionOtherOptions, counts);
		}
	}
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	CloseFile(outputFile);

}
		
void WriteLHMPNumber(int outputFile, short options, long counts, long channel)
{
	char lineToOutput[300];
	
	if ((options & kFLOATINGCOUNTS) == 0)
	{
		sprintf(lineToOutput, "%8d", (int)counts);	
	}
	else
	{
		sprintf(lineToOutput, "%8.02f", counts);
	}	
	WriteFile(outputFile, lineToOutput, strlen(lineToOutput));
	if ((channel+1)%10 == 0)
	{
		WriteFile(outputFile, "\n", 1);
	}
}

void writeLHMPFile(convertParamBlock *paramBlock, calPoint *dataPoints, void* header)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	double tTheta;
	double counts;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	double min, max;
	
	if (paramBlock->inputFileType == kCHN)
	{
		size = ((MCAFileHeader *)header)->num_chans; 
	}
	else
	{
		size = ((GUFIHeader*)header)->numPoints;
	}
	findAFreeFileName(paramBlock->outputFileName, "dat", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		return;	
	}
	if ((paramBlock->conversionOtherOptions & kCONSTANTSTEP)!=0)
	{
		setUpForConstantStep(dataPoints, header, paramBlock);
		step = constStepOpts.stepValue;
		minTheta = dataPoints[0].xPoint;
		if (minTheta < theRangeOptions.startValue && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
		{
			minTheta = theRangeOptions.startValue;
		}
		maxChannel = (int)(dataPoints[size-1].xPoint-minTheta)/step;
	}
	else
	{
		step = 1;
		minTheta = 0;
		maxChannel = size;
	}	
	GetHeaderInfo(paramBlock, step, minTheta, maxChannel, &min, &max, size);
	writeLHMPHeader(min, step, max, outputFile);
	for (channel =0; channel<maxChannel; channel++)
	{
		updateProgress(channel, maxChannel, paramBlock->progressPanelHandle);
		tTheta = minTheta + step * channel;
		if (0 == getTThetaAndCounts(&tTheta, &counts, dataPoints, header, paramBlock))
		{	
			WriteLHMPNumber(outputFile, paramBlock->conversionOtherOptions, counts, channel);
		}
	}
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	CloseFile(outputFile);
}

void convertAnyFile(convertParamBlock *paramBlock)
{
	calPoint *dataPoints = NULL;
	MCAFileHeader MCAHeader;
	GUFIHeader	GUFIFileHeader;
	void * header;
	
	if (paramBlock->inputFileType == kGUFI)
	{
		if (readGUFIFile(paramBlock, &dataPoints, &GUFIFileHeader)!=0)
		{
			return;
		}
		header = &GUFIFileHeader;
	}
	else
	{
		if (readMCAFile(paramBlock, &dataPoints, &MCAHeader)!=0)
		{
			return;
		}
		header = &MCAHeader;
	}
	if (paramBlock->conversionType == 0)
	{
		writeTextFileChannels(paramBlock, &MCAHeader, dataPoints);
	}
	else
	{
		switch (paramBlock->fileOutputType)
		{
			case kTEXT:
				writeTextFileTTheta(paramBlock, dataPoints, header);
			break;
			case kXFIT:
				writeXFITFile(paramBlock, dataPoints, header);
			break;
			case kCPI:
			    writeCPIFile(paramBlock, dataPoints, header);
			break;
			case kLHMP:
				writeLHMPFile(paramBlock, dataPoints, header);
			break;
		}
	} 
	if (dataPoints != NULL)
	{
		free(dataPoints);
	}
}

/****************************ConvertFiles**************************/
/* This displays the choose file name box if its a single file to */
/* be converted or the directory window if its multipule files.	  */ 
/* Then converts all the files using convert.					  */
/*	Parameter:													  */
/*			parentPanel: This is a handle to the panel which this */
/*			method was called from.								  */
/******************************************************************/
void convertFiles(int parentPanel)
{
	convertParamBlock paramBlock;  //The paramater block which is to be sent to the convert any method.
	
	int fileSelect;  //if the OK button on the file select button ws pressed
	int numOfFiles;  //The number of files which need converting
	int index;		 //The current file which it's up to.
	int stringIndex; //A pointer to one of the strings in the file
	char errorMessage[30]; //any error message which needs outputing
	char outputDir[300]; //The directory selected for output of the files
	char fileOf[20]="File 1 of 1"; //A string which displays on the progress window what file there up to.
	char directoryPath[300]; //The default directory for the files to be output into
	char outputFileName[300];
	char inputFileName[300];
	char fileExtesion[7];
	int fileType;
	int tempValue;
	
	paramBlock.conversionType = 0;
	paramBlock.conversionOtherOptions = 0;
	paramBlock.fileOutputType = 0;
	paramBlock.addExtension = 0;
	paramBlock.progressPanelHandle = 0;
	paramBlock.parentPanelHandle = 0;
	paramBlock.parentPanelHandle = parentPanel;
	
	if ((paramBlock.progressPanelHandle = LoadPanel(0, "DiffConv.uir", kPROGRESS))<0) // load in the progress panel and check that it was loaded in.
	{	//If it wasn't...
		MessagePopup("Panel Error", "There was an error will loading the progress panel"); //Tell the user
		DisplayPanel(parentPanel); //Bring the main panel to the front
		DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
		return; //Leave it
	}
	GetCtrlVal(parentPanel, MP_kCONVERT, &tempValue);
	if (tempValue==0)
	{
		GetCtrlAttribute(parentPanel, MP_kOUTPUTFILETYPE, ATTR_DIMMED, &tempValue);
		paramBlock.conversionType = 0;
		if (tempValue == 0)
		{
			GetCtrlVal(parentPanel, MP_kOUTPUTFILETYPE, &paramBlock.fileOutputType);
		}
	}
	else
	{   
		paramBlock.conversionOtherOptions = 0;
		GetCtrlVal(parentPanel, MP_kROUNDTOINT, &tempValue); 
		if (tempValue == 1)
		{
			paramBlock.conversionOtherOptions |= kROUNDTOINT;	
		}
		GetCtrlVal(parentPanel, MP_kCONSTANTSTEP, &tempValue); 
		if (tempValue == 1)
		{
			paramBlock.conversionOtherOptions |= kCONSTANTSTEP;	
		} 
		GetCtrlVal(parentPanel, MP_kLIMMITRANGECB, &tempValue); 
		if (tempValue == 1)
		{
			paramBlock.conversionOtherOptions |= kLIMITRANGE;	
		}
		if (constStepOpts.outputFloatingCounts == 1)
		{
			paramBlock.conversionOtherOptions |= kFLOATINGCOUNTS;
		}
		GetCtrlVal(parentPanel, MP_kOUTPUTFILETYPE, &paramBlock.fileOutputType);
		GetCtrlVal(parentPanel, MP_kLINEARCONVERTION, &tempValue); //Get the value of the linear convertion radio button
		if (tempValue == 1)
		{
			paramBlock.conversionType = kTWOTHETALINEAR;
		}
		else
		{
			GetCtrlVal(parentPanel, MP_kSPLINECONVERTION, &tempValue); //Get the value of the Spline Convertion readio button.
			if (tempValue == 1)
			{
				paramBlock.conversionType = kTWOTHETASPLINE;
			}
			else
			{
				paramBlock.conversionType = kNULLCONVERSION;
			}
		}
	}
	switch (paramBlock.fileOutputType)
	{
		case kTEXT:
			strcpy(fileExtesion, "*.txt"); 
		break;
		case kXFIT:
			strcpy(fileExtesion, "*.dat"); 
		break;
		case kCPI:
			strcpy(fileExtesion, "*.cpi"); 
		break;
		case kLHMP:
			strcpy(fileExtesion, "*.dat"); 
		break;
		case kGUFI:
			strcpy(fileExtesion, "*.guf"); 
		break;
	}
	GetLabelFromIndex(parentPanel, MP_kFILES, 0, directoryPath);  //Get the path for the first file in the list
	getDirPath(directoryPath);
	GetNumListItems(parentPanel, MP_kFILES, &numOfFiles); //find the number of files in the list
	if (numOfFiles==1) //If the number of files is one then.
	{
		fileSelect = FileSelectPopup (directoryPath, fileExtesion, fileExtesion, "Select Output File",
			   VAL_OK_BUTTON, 0, 1, 1, 0, outputFileName);  //Get the output file name.
		if (fileSelect==0)  //if the user canceled from the file select popup then
		{
			DisplayPanel(parentPanel); //bring the main panel to the front
			DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
			return; //leave this.
		}
	}
	else
	{
		fileSelect = DirSelectPopup (directoryPath, "Select Output Directory", 1, 1, outputDir);  //Let the user pick where they want to put the converted files.
		if (fileSelect==0)
		{
			DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
			DisplayPanel(parentPanel); //bring the main panel to the front
			return;   //leave this place.
		}
	}
	DisplayPanel(paramBlock.progressPanelHandle);
	for (index = 0; index < numOfFiles; index++)
	{
		GetLabelFromIndex(parentPanel, MP_kFILES, index, inputFileName);
		GetValueFromIndex(parentPanel, MP_kFILES, index, &fileType);
		SetCtrlVal(paramBlock.progressPanelHandle, kPROGRESS_kINPUTFILENAME, inputFileName);
		if (numOfFiles > 1)
		{
			stringIndex= strlen(inputFileName);
			while (inputFileName[stringIndex]!='\\' && inputFileName[stringIndex]!=':' && stringIndex > 0)  //While stringIndex isn't pointing to a \ char then
			{
				stringIndex--;  //Move to the previose char
			} 
			sprintf(outputFileName, "%s%s", outputDir , (char *)(inputFileName+stringIndex));
		}
		removeExtension(outputFileName);
		sprintf(fileOf, "File %d of %d", index+1, numOfFiles);	//Create a sting which says how many files are still to be converted
		SetCtrlVal(paramBlock.progressPanelHandle, kPROGRESS_kFILENUMLABEL, fileOf);
		paramBlock.inputFileName = inputFileName;
		paramBlock.outputFileName = outputFileName;
		paramBlock.addExtension = 1;
		paramBlock.inputFileType = fileType;
		spline(calPoints, header.numOfCalibPoints, 0, 1);
		convertAnyFile(&paramBlock);
	}
	HidePanel(paramBlock.progressPanelHandle);  //Hide the progress panel
	DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
	DisplayPanel(parentPanel); //bring the main panel to the front
}
//----------------------------------------------------------------- 
