#include <utility.h>
#include <userint.h>
#include "Conversion.h"
#include "MCAtoTEXT.h"
#include "MCAtoTEXTTypes.h"
#include "Options.h"
#include "RangeOptPanel.h" 
#include "SplineOptionPanel.h"
#include "ConstantStepOptPanel.h"
#include "CPIOps.h" 

#define CHN -1// This is the number used to check that the opened file is a CHN file or not
#define kNUMOFPOINTSFORCONSSTEP 5

double startAngle;
double inputStep;

void convertAnyFile(convertParamBlock *paramBlock);
void removeExtension(char *string);
void writeTextFileChannels(convertParamBlock *paramBlock);
void addExtension(char *string, char extension[3], int maxStringLen);
void findAFreeFileName(char *filePathWOExt, char extension[3], char *outputFileName, int maxLenOfOutputFilePath);
void writeTextFileTTheta(convertParamBlock *paramBlock);
void setConstStepVars(convertParamBlock *paramBlock, long *channelData, float *intensityData, short conversionType, calPoint *constStepCalPoints, MCAFileHeader *MCAHeader, GUFIHeader *GUFIFileHeader, int progressPanel);
void writeXFITHeader(double start, double step, double stopVal, int fileHandle);
void writeCPIHeader(double start, double step, double stopVal, int fileHandle);
void writeLHMPHeader(double start, double step, double stopVal, int fileHandle);
void writeXFITFile(convertParamBlock *paramBlock);
void writeCPIFile(convertParamBlock *paramBlock);
void writeLHMPFile(convertParamBlock *paramBlock);

double round(double number)
{
	double tempNumber;
	
	tempNumber = (int)(number/1);
	if ((number - tempNumber)>=0.5)
	{
		tempNumber++;
	}
	return tempNumber;
}

void getDirPath(char dirPath[])
{
	int stringIndex=0; //initalize the string index
	
	while (dirPath[stringIndex]!=0x00) //While it's not pointing to the end of the file
	{
		stringIndex++; //Move to the next char in the string
	}
	while (dirPath[stringIndex]!='\\') //While stringIndex isn't pointing to a \ then
	{
		stringIndex--; //Go to the previose char
	}
	dirPath[stringIndex+1]=0x00;  //cut the file name of the end of the path.
}

/*****************************convertToTTheta***********************/
/* This converts the channel number entered into the 2-theta	   */
/* This convert to 2-theta using either a linear relation ship of a*/
/* spline curve depending no the value of the linear convertion	   */
/* radio button.												   */
/*  Parameters:													   */
/*		channelNum: this is the value which is to be converted to  */
/*	   2-theta.													   */
/*	returns:													   */
/*		It returns the 2-theta value which corrasponds to the      */
/*     channel number past to it.								   */
/*******************************************************************/
double channelToTTheta(double channelNum, int linearValue)
{
	double tempValue; //Value for extending th spline curve.
	
	if (linearValue) //If the linear convertion radio button is on then...
	{
		return header.K+header.C*channelNum;  //Do the linear convertion.
	}
	else   //If the linear convertion radio button is off then...
	{
	 	if (calPoints[0].xPoint>channelNum || calPoints[header.numOfCalibPoints-1].xPoint<channelNum) //If the cahnnel number is out of the cal points range then...
	 	{
	 		tempValue = -(calPoints[header.numOfCalibPoints-1].yPoint-calPoints[0].yPoint)/(calPoints[header.numOfCalibPoints-1].xPoint-calPoints[0].xPoint);  //Work out the gradaint of the line between the first and last calibration points.
	 		return (calPoints[0].xPoint-channelNum)*tempValue+calPoints[0].yPoint;   //Uses a linear relationship to calculate the 2-theta value and return that value.
	 	}
	 	//else if the point is with in the range of the calibration points then...
		return splint(calPoints, header.numOfCalibPoints, channelNum);  //Uses spline to work out the 2-theta value.  
	}
	return 0;
}

//----------------------------------------------------------------- 
//----------ConvertFiles-------------------------------------------
// This displays the choose file name box if its a single file to 
// be converted or the directory window if its multipule files. 
// Then converts all the files using convert.
void convertFiles(int parentPanel)
{
	convertParamBlock paramBlock;  //The paramater block which is to be sent to the convert any method.
	
	int fileSelect;  //if the OK button on the file select button ws pressed
	int numOfFiles;  //The number of files which need converting
	int index;		 //The current file which it's up to.
	int stringIndex; //A pointer to one of the strings in the file
	char errorMessage[30]; //any error message which needs outputing
	char outputDir[300]; //The directory selected for output of the files
	char fileOf[20]="File 1 of 1"; //A string which displays on the progress window what file there up to.
	char directoryPath[300]; //The default directory for the files to be output into
	char outputFileName[300];
	char inputFileName[300];
	char fileExtesion[7];
	int fileType;
	int tempValue;
	
	paramBlock.conversionType = 0;
	paramBlock.conversionOtherOptions = 0;
	paramBlock.fileOutputType = 0;
	paramBlock.addExtension = 0;
	paramBlock.progressPanelHandle = 0;
	paramBlock.parentPanelHandle = 0;
	paramBlock.parentPanelHandle = parentPanel;
	
	if ((paramBlock.progressPanelHandle = LoadPanel(0, "MCAtoTEXT.uir", kPROGRESS))<0) // load in the progress panel and check that it was loaded in.
	{	//If it wasn't...
		MessagePopup("Panel Error", "There was an error will loading the progress panel"); //Tell the user
		DisplayPanel(parentPanel); //Bring the main panel to the front
		DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
		return; //Leave it
	}
	GetCtrlVal(parentPanel, MP_kTWOTHETA, &tempValue);
	if (tempValue==0)
	{
		paramBlock.conversionType == 0;
	}
	else
	{   
		paramBlock.conversionOtherOptions = 0;
		GetCtrlVal(parentPanel, MP_kROUNDTOINT, &tempValue); 
		if (tempValue == 1)
		{
			paramBlock.conversionOtherOptions |= kROUNDTOINT;	
		}
		GetCtrlVal(parentPanel, MP_kCONSTANTSTEP, &tempValue); 
		if (tempValue == 1)
		{
			paramBlock.conversionOtherOptions |= kCONSTANTSTEP;	
		} 
		GetCtrlVal(parentPanel, MP_kLIMMITRANGECB, &tempValue); 
		if (tempValue == 1)
		{
			paramBlock.conversionOtherOptions |= kLIMITRANGE;	
		}
		if (constStepOpts.outputFloatingCounts == 1)
		{
			paramBlock.conversionOtherOptions |= kFLOATINGCOUNTS;
		}
		GetCtrlVal(parentPanel, MP_kOUTPUTFILETYPE, &paramBlock.fileOutputType);
		GetCtrlVal(parentPanel, MP_kLINEARCONVERTION, &tempValue); //Get the value of the linear convertion radio button
		if (tempValue == 1)
		{
			paramBlock.conversionType = 1;
		}
		else
		{
			paramBlock.conversionType = 2;
		}
	}
	switch (paramBlock.fileOutputType)
	{
		case kTEXT:
			strcpy(fileExtesion, "*.txt"); 
		break;
		case kXFIT:
			strcpy(fileExtesion, "*.dat"); 
		break;
		case kCPI:
			strcpy(fileExtesion, "*.cpi"); 
		break;
		case kLHMP:
			strcpy(fileExtesion, "*.dat"); 
		break;
		case kGUFI:
			strcpy(fileExtesion, "*.guf"); 
		break;
	}
	GetLabelFromIndex(parentPanel, MP_kFILES, 0, directoryPath);  //Get the path for the first file in the list
	getDirPath(directoryPath);
	GetNumListItems(parentPanel, MP_kFILES, &numOfFiles); //find the number of files in the list
	if (numOfFiles==1) //If the number of files is one then.
	{
		fileSelect = FileSelectPopup (directoryPath, fileExtesion, fileExtesion, "Select Output File",
			   VAL_OK_BUTTON, 0, 1, 1, 0, outputFileName);  //Get the output file name.
		if (fileSelect==0)  //if the user canceled from the file select popup then
		{
			DisplayPanel(parentPanel); //bring the main panel to the front
			DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
			return; //leave this.
		}
	}
	else
	{
		fileSelect = DirSelectPopup (directoryPath, "Select Output Directory", 1, 1, outputDir);  //Let the user pick where they want to put the converted files.
		if (fileSelect==0)
		{
			DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
			DisplayPanel(parentPanel); //bring the main panel to the front
			return;   //leave this place.
		}
	}
	DisplayPanel(paramBlock.progressPanelHandle);
	for (index = 0; index < numOfFiles; index++)
	{
		GetLabelFromIndex(parentPanel, MP_kFILES, index, inputFileName);
		GetValueFromIndex(parentPanel, MP_kFILES, index, &fileType);
		SetCtrlVal(paramBlock.progressPanelHandle, kPROGRESS_kINPUTFILENAME, inputFileName);
		if (numOfFiles > 1)
		{
			stringIndex= strlen(inputFileName);
			while (inputFileName[stringIndex]!='\\' && inputFileName[stringIndex]!=':' && stringIndex > 0)  //While stringIndex isn't pointing to a \ char then
			{
				stringIndex--;  //Move to the previose char
			} 
			sprintf(outputFileName, "%s%s", outputDir , (char *)(inputFileName+stringIndex));
		}
		removeExtension(outputFileName);
		sprintf(fileOf, "File %d of %d", index+1, numOfFiles);	//Create a sting which says how many files are still to be converted
		SetCtrlVal(paramBlock.progressPanelHandle, kPROGRESS_kFILENUMLABEL, fileOf);
		paramBlock.inputFileName = inputFileName;
		paramBlock.outputFileName = outputFileName;
		paramBlock.addExtension = 1;
		paramBlock.inputFileType = fileType;
		convertAnyFile(&paramBlock);
	}
	HidePanel(paramBlock.progressPanelHandle);  //Hide the progress panel
	DiscardPanel(paramBlock.progressPanelHandle); //Remove the progress panel from memory
	DisplayPanel(parentPanel); //bring the main panel to the front
}
//----------------------------------------------------------------- 

/************************removeExtension****************************/
/*  This removes the extension of a file name if it has one.       */
/* Parameters:													   */
/*      string: The path anem which is to have the extension       */
/*	removed from.	   											   */
/*******************************************************************/
void removeExtension(char *string)
{
	int indexInString = 0;
	indexInString = strlen(string);
	while (string[indexInString]!='.' && string[indexInString]!='\\' && string[indexInString]!=':' && indexInString>0)
	{
		indexInString--;
	}
	if (string[indexInString]=='.')
	{
		string[indexInString]=0x00;
	}
}

void convertAnyFile(convertParamBlock *paramBlock)
{
	if (paramBlock->conversionType == 0)
	{
		writeTextFileChannels(paramBlock);
	}
	else
	{
		switch (paramBlock->fileOutputType)
		{
			case kTEXT:
				writeTextFileTTheta(paramBlock);
			break;
			case kXFIT:
				writeXFITFile(paramBlock);
			break;
			case kCPI:
			    writeCPIFile(paramBlock);
			break;
			case kLHMP:
				writeLHMPFile(paramBlock);    
			break;
		}
	} 
}

void addExtension(char *string, char extension[3], int maxStringLen)
{
	int stringLen = strlen(string);
	int indexInExt = 0;
	
	string[stringLen]='.';
	stringLen++;
	while (indexInExt < 4 && extension[indexInExt]!=0x00 && stringLen < maxStringLen)
	{
		string[stringLen]=extension[indexInExt];
		indexInExt++;
		stringLen++;
	}
	string[stringLen]=0x00;
}

void findAFreeFileName(char *filePathWOExt, char extension[3], char *outputFileName, int maxLenOfOutputFilePath)
{
	char message[300];
	int outputFile;
	int fileUpTo = 0;
	int numOfZeros = 0;
	
	DisableBreakOnLibraryErrors();
	strcpy(outputFileName, filePathWOExt);
	addExtension(outputFileName, extension, maxLenOfOutputFilePath); //just add the extension .txt to the end of the file.
	if ((outputFile=OpenFile(outputFileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))>=00) //trys to open the file as read only to see if that file already exists.
	{
		sprintf(message, "Do you want to repalce\nthe file: %s?", outputFileName);  //Add the file name of the output file onto the other string and put the result into message
		Delay(0.1);
		if (!ConfirmPopup("Replace?", message))
		{
			do
			{
				fileUpTo++;
				strcpy(outputFileName, filePathWOExt);
				sprintf(outputFileName, "%s-", outputFileName);
				for (numOfZeros = 3-(int)(log(fileUpTo)); numOfZeros > 0; numOfZeros--)
				{
					sprintf(outputFileName, "%s0", outputFileName);
				}
				sprintf(outputFileName, "%s%d", outputFileName, fileUpTo);
				addExtension(outputFileName, extension, maxLenOfOutputFilePath); //just add the extension .txt to the end of the file.
			}
			while ((outputFile=OpenFile(outputFileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))>=0 && fileUpTo < 9999);
			if (outputFile>=0)
			{
				CloseFile(outputFile);
			}
		}
	}
	EnableBreakOnLibraryErrors();	     
}


int readMCAFile(char *MCAFileName, long **data, MCAFileHeader *MCAHeader)
{
	char message[300];
	int inputFile;
	long numOfBytesRead;
	
	if ((inputFile=OpenFile(MCAFileName, VAL_READ_ONLY, VAL_APPEND, VAL_BINARY))<0)
	{
		sprintf(message, "Couldn't open input file: %s\nFile may not exist.\nWill skip conversion.", MCAFileName);
		MessagePopup("File Error", message);
		return -1;
	}
	numOfBytesRead = ReadFile(inputFile, ((char*)(MCAHeader)), sizeof(*MCAHeader)); //read in the file header for the input file
	if (MCAHeader->fileType!=CHN)	//Check that it's got the correct file type
	{
		sprintf(message, "The input file \"%s\" \nwasn't a CHN file.", MCAFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	if (*data!=NULL)
	{
		free(*data);
		*data = NULL;
	}
	*data = malloc(sizeof(long)*MCAHeader->num_chans);  //allocate enough memory to load the while file into memory
	numOfBytesRead = ReadFile(inputFile, (char *)*data, sizeof(long)*MCAHeader->num_chans);
	if (numOfBytesRead != sizeof(long)*MCAHeader->num_chans)
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", MCAFileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	CloseFile(inputFile);
	startAngle =0;
	inputStep =1;
	return 0;
}

int readGUFIFile(char *fileName, float **data, GUFIHeader *header)
{
	char message[300];
	int inputFile;
	long numOfBytesRead, bytes=1;
	short i;
	
	if ((inputFile=OpenFile(fileName, VAL_READ_ONLY, VAL_APPEND, VAL_BINARY))<0)
	{
		sprintf(message, "Couldn't open input file: %s\nFile may not exist.\nWill skip conversion.", fileName);
		MessagePopup("File Error", message);
		return -1;									   
	}
	//numOfBytesRead = ReadFile(inputFile, ((char*)(header)), sizeof(GUFIHeader)); //read in the file header for the input file
	SetFilePtr(inputFile, 0x09, 0);
	numOfBytesRead = ReadFile(inputFile, (char*)&(header->startAng), sizeof(float));
	if (numOfBytesRead != sizeof(float))
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", fileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	SetFilePtr(inputFile, 0x0D, 0);
	numOfBytesRead = ReadFile(inputFile, (char*)&(header->stepAng), sizeof(float));
	if (numOfBytesRead != sizeof(float))
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", fileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	SetFilePtr(inputFile, 0x11, 0);
	numOfBytesRead = ReadFile(inputFile, (char*)&(header->numPoints), sizeof(short));
	if (*data!=NULL)
	{
		free(*data);
		*data = NULL;
	}
	SetFilePtr(inputFile, 0x283, 0);
	*data = malloc(sizeof(float)*header->numPoints);
	numOfBytesRead = 0;
	for (i=0; i<header->numPoints && bytes > 0; i++)
	{
		numOfBytesRead += bytes = ReadFile(inputFile, (char *)&(*data)[i], sizeof(float));
	}
	if (numOfBytesRead != sizeof(float)*header->numPoints)
	{
		sprintf(message, "The input file \"%s\" \nhas been corrupted.", fileName);
		MessagePopup("Error", message);  //If the file type is not correct then tell the user
		return -1;  //And leg it.
	}
	CloseFile(inputFile);
	startAngle =header->startAng;
	inputStep = header->stepAng;
	return 0;
}

void writeTextFileChannels(convertParamBlock *paramBlock)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	long *channelData = NULL;
	int progressValue=0;
	char lineToOutput[300];
	
	MCAFileHeader MCAHeader;
	
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName);
	if (readMCAFile(paramBlock->inputFileName, &channelData, &MCAHeader)<0)
	{
		if (channelData!=NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;
	}
	findAFreeFileName(paramBlock->outputFileName, "txt", outputFileName, 300);
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full or you may not have write permission.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	WriteLine(outputFile, "Channel\tCount", 13);
	for (channel =0; channel<MCAHeader.num_chans; channel++)
	{
		progressValue = (int)((float)((float)channel/(float)MCAHeader.num_chans)*100); //work out the amount of the file which has been processed.
		if ((progressValue)%10==0) //if the value is divisible by 10 then...
		{
			SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
		}
		lineToOutput[0]=0x00;
		sprintf(lineToOutput, "%d	%d", channel+MCAHeader.chan_offset, channelData[channel]);	//conver the channel number to 2-theta and construct a line with the 2-theta value and the intensity on it.
		WriteLine(outputFile, lineToOutput, strlen(lineToOutput));
	}
	CloseFile(outputFile);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	free(channelData);
	channelData = NULL;
}

double findCountForTThetaRebin(calPoint *calPoints, long numOfChans, double tThetaVal) 
{
	return 0;	
}

double findCountForTThetaLin(calPoint *calPoints, long numOfChans, double stepSize, double tThetaVal)
{
	int start=0, end = numOfChans-1, index = (end-start)/2;
	double temp;
	
	while ((calPoints[index].xPoint != tThetaVal) && start!=end && start +1 != end)
	{
		if (calPoints[index].xPoint<tThetaVal)
		{
			start = index;	
		}
		else
		{
			end =index;	
		}
		index = (end-start)/2+start;
	}
	if (calPoints[index].xPoint == tThetaVal)
	{
		return calPoints[index].yPoint; 
	}
	else if (calPoints[index].xPoint > tThetaVal)
	{
		return (calPoints[index].yPoint-calPoints[index-1].yPoint)/(calPoints[index].xPoint-calPoints[index-1].xPoint)*(tThetaVal -calPoints[index-1].xPoint)+calPoints[index].yPoint;	
	}
	else
	{
		return (calPoints[index+1].yPoint-calPoints[index].yPoint)/(calPoints[index+1].xPoint-calPoints[index].xPoint)*(tThetaVal -calPoints[index].xPoint)+calPoints[index].yPoint;	
	}
	return -1000;
}

int getTThetaAndCountsGUFI(float *intensityData, int channelNum, double *tTheta, double *counts, short conversionType, short conversionOtherOptions, GUFIHeader *GUFIFileHeader, calPoint *constCalPoints)
{
	switch (constStepOpts.conversionMethod)
	{
		case kSPLINE: 
			*counts = splint(constCalPoints, GUFIFileHeader->numPoints, *tTheta);
		break;
		case kLINEAR:
			*counts = findCountForTThetaLin(constCalPoints, GUFIFileHeader->numPoints, GUFIFileHeader->stepAng, *tTheta);	
		break;
	}
	if ((conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((*tTheta < theRangeOptions.startValue) || (*tTheta > theRangeOptions.endValue))
		{
			return -1;
		}
	}
	if ((conversionOtherOptions & kROUNDTOINT) != 0)
	{
		
		*tTheta = round(*tTheta);
	}
	if ((conversionOtherOptions & kFLOATINGCOUNTS) == 0)
	{
		*counts = round(*counts);
	}
	return 0;
}

int getTThetaAndCountsCHN(long *channelData, long channelNum, double *tTheta, double *counts, short conversionType, short conversionOtherOptions, MCAFileHeader *MCAHeader, calPoint *constCalPoints)
{
	if ((conversionOtherOptions & kCONSTANTSTEP) == 0)
	{
		*tTheta = channelToTTheta(channelNum, (kTWOTHETALINEAR & conversionType));
		*counts = channelData[channelNum];
	}
	else
	{
		switch (constStepOpts.conversionMethod)
		{
			case kSPLINE: 
				*counts = splint(constCalPoints, MCAHeader->num_chans, *tTheta);
			break;
			case kLINEAR:
				*counts = findCountForTThetaLin(constCalPoints, MCAHeader->num_chans, 1, *tTheta);	
			break;
		}
	}
	if ((conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((*tTheta < theRangeOptions.startValue) || (*tTheta > theRangeOptions.endValue))
		{
			return -1;
		}
	}
	if ((conversionOtherOptions & kROUNDTOINT) != 0)
	{
		
		*tTheta = round(*tTheta);
	}
	if ((conversionOtherOptions & kFLOATINGCOUNTS) == 0)
	{
		*counts = round(*counts);
	}
	return 0;
}

int readFile(convertParamBlock  *paramBlock, long **channelData, float **intensityData, MCAFileHeader *MCAHeader, GUFIHeader *GUFIFileHeader)
{
	if (paramBlock->inputFileType == kCHN)
	{
		if (readMCAFile(paramBlock->inputFileName, channelData, MCAHeader)<0)
		{
			if (*channelData!=NULL)
			{
				free(*channelData);
				channelData = NULL;
			}
			return -1;
		}
	}
	else
	{
		if (readGUFIFile(paramBlock->inputFileName, intensityData, GUFIFileHeader))
		{
			if (*intensityData!=NULL)
			{
				free(*intensityData);
				*intensityData = NULL;
			}
			return -1;	
		}
	}
	return 0;
}
	
void writeTextFileTTheta(convertParamBlock *paramBlock)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	long *channelData = NULL;
	float *intensityData = NULL;
	int progressValue=0;
	char lineToOutput[300];
	double tTheta;
	double counts;
	calPoint *constStepCalPoints = NULL;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	int error;
	
	MCAFileHeader MCAHeader;
	GUFIHeader GUFIFileHeader;
	
	constStepCalPoints = NULL;
	if (paramBlock->conversionType == kTWOTHETASPLINE); 
	{
		spline(calPoints, header.numOfCalibPoints, 0);
	}
	if (readFile(paramBlock, &channelData, &intensityData, &MCAHeader, &GUFIFileHeader) != 0)
	{
		if (constStepCalPoints!=NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		return;
	}
	if (paramBlock->inputFileType == kCHN)
	{
		size = MCAHeader.num_chans; 
	}
	else
	{
		size = GUFIFileHeader.numPoints;
	}
	if (constStepCalPoints)
	{
		free(constStepCalPoints);
		constStepCalPoints = NULL;
	}
	constStepCalPoints = malloc(size*sizeof(calPoint)); 
	if (constStepCalPoints==NULL)
	{
		MessagePopup("No Memory", "There isn't enough memory to do this!");
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	findAFreeFileName(paramBlock->outputFileName, "txt", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	WriteLine(outputFile, "2-Theta\tCount", 13);
	if ((paramBlock->conversionOtherOptions & kCONSTANTSTEP)>0)
	{
		setConstStepVars(paramBlock, channelData, intensityData, paramBlock->conversionType, constStepCalPoints, &MCAHeader, &GUFIFileHeader, paramBlock->progressPanelHandle);
		step = constStepOpts.stepValue;
		minTheta = channelToTTheta(startAngle, (kTWOTHETALINEAR & paramBlock->conversionType));
		maxChannel = (int)((channelToTTheta((size-1)*inputStep, (kTWOTHETALINEAR & paramBlock->conversionType))-minTheta)/step);
	}
	else
	{
		step = 1;
		minTheta = 0;
		maxChannel = size;
	}
	for (channel =0; channel<maxChannel; channel++)
	{
		progressValue = (int)((float)((float)channel/(float)maxChannel)*100); //work out the amount of the file which has been processed.
		if ((progressValue)%10==0) //if the value is divisible by 10 then...
		{
			SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
		}
		lineToOutput[0]=0x00;
		tTheta = minTheta + step * channel;
		if (paramBlock->inputFileType == kCHN)
		{
			error = getTThetaAndCountsCHN(channelData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &MCAHeader, constStepCalPoints);
		}
		else
		{
			error = getTThetaAndCountsGUFI(intensityData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &GUFIFileHeader, constStepCalPoints);
		}
		if (0 == error)
		{
			if ((paramBlock->conversionOtherOptions & kROUNDTOINT) != 0)
			{
				sprintf(lineToOutput, "%d	", (int)tTheta);	//convert the channel number to 2-theta and construct a line with the 2-theta value and the intensity on it.
			}
			else
			{	
				sprintf(lineToOutput, "%9.08f	", tTheta);	//convert the channel number to 2-theta and construct a line with the 2-theta value and the intensity on it.
			}	
			if ((paramBlock->conversionOtherOptions & kFLOATINGCOUNTS) == 0)
			{
				sprintf(lineToOutput, "%s%d", lineToOutput, (int)counts);
			}
			else
			{
				sprintf(lineToOutput, "%s%8.04f", lineToOutput, counts); 	
			}
			WriteLine(outputFile, lineToOutput, strlen(lineToOutput));
		}
	}
	CloseFile(outputFile);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	if (constStepCalPoints != NULL)
	{
		free(constStepCalPoints);
		constStepCalPoints = NULL;
	}
	if (channelData != NULL)
	{
		free(channelData);
		channelData = NULL;
	}
}

void setConstStepVars(convertParamBlock *paramBlock, long *channelData, float *intensityData, short conversionType, calPoint *constStepCalPoints, MCAFileHeader *MCAHeader, GUFIHeader *GUFIFileHeader, int progressPanel)
{
	long channel;
	double yPoint;
	long size;
	
	if (paramBlock->inputFileType == kCHN)
	{
		for (channel = 0; channel < MCAHeader->num_chans; channel++)
		{
			constStepCalPoints[channel].xPoint = channelToTTheta(channel, constStepOpts.conversionMethod!=kSPLINE );
			constStepCalPoints[channel].yPoint = channelData[channel];
		}
		size = MCAHeader->num_chans;
	}
	else
	{
		for (channel = 0; channel < GUFIFileHeader->numPoints; channel++)
		{
			constStepCalPoints[channel].xPoint = channelToTTheta(channel*GUFIFileHeader->stepAng+GUFIFileHeader->startAng, constStepOpts.conversionMethod!=kSPLINE);
			constStepCalPoints[channel].yPoint = intensityData[channel];
		}
		size = GUFIFileHeader->numPoints;
	}
	if (constStepOpts.conversionMethod==kSPLINE)
	{				  
		spline(constStepCalPoints, size, progressPanel); 
	}
}

void writeXFITHeader(double start, double step, double stopVal, int fileHandle)
{
	char lineToOutput[300];
	WriteLine(fileHandle, "\n\n\n", 3);
	sprintf(lineToOutput, "%8.3f%8.3f%8.3f", start, step, stopVal);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
}

void writeCPIHeader(double start, double step, double stopVal, int fileHandle)
{
	char lineToOutput[300];
	
	WriteLine(fileHandle, "SIETRONICS XRD SCAN", 19);
	sprintf(lineToOutput, "%0.03f", start);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput)); 
	sprintf(lineToOutput, "%0.03f", stopVal);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput)); 
	sprintf(lineToOutput, "%0.03f", step); 
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
	WriteLine(fileHandle, CPIOpt.anodeSymbol, 2);
	sprintf(lineToOutput, "%0.5f", CPIOpt.waveLength);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
	WriteLine(fileHandle, DateStr(), strlen(DateStr()));
	WriteLine(fileHandle, "1", 1);
	WriteLine(fileHandle, "MCAtoOTHER", 10);
	WriteLine(fileHandle, "SCANDATA", 9);
}

void writeLHMPHeader(double start, double step, double stopVal, int fileHandle)
{
	char lineToOutput[300];
	sprintf(lineToOutput, "%8.3f%8.3f%8.3f", start, step, stopVal);
	WriteLine(fileHandle, lineToOutput, strlen(lineToOutput));
}

void GetHeaderInfo(convertParamBlock *paramBlock, double *step, double *minTheta, long *maxChannel, double *min, double *max, int numOfPoints)
{
	*step = constStepOpts.stepValue;
	*minTheta = channelToTTheta(startAngle, (kTWOTHETALINEAR & paramBlock->conversionType));
	*maxChannel = (int)((channelToTTheta((numOfPoints-1)*inputStep, (kTWOTHETALINEAR & paramBlock->conversionType))-(*minTheta))/(*step));
	if (*minTheta < theRangeOptions.startValue && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((((theRangeOptions.startValue-(*minTheta))/(*step))-(int)((theRangeOptions.startValue-(*minTheta))/(*step))) != 0)
		{
			*min = (int)(((theRangeOptions.startValue-(*minTheta))/(*step))+1)*(*step)+(*minTheta);
		}
		else
			*min = theRangeOptions.startValue;
	}
	else
	{
		*min = *minTheta;
	}
	if (*maxChannel*(*step)+(*minTheta) > theRangeOptions.endValue  && (paramBlock->conversionOtherOptions & kLIMITRANGE) != 0)
	{
		if ((((theRangeOptions.endValue-(*minTheta))/(*step))-(int)((theRangeOptions.endValue-(*minTheta))/(*step))) != 0)
		{
			*max = ((int)((theRangeOptions.endValue-*minTheta)/(*step)))*(*step)+(*minTheta);
		}
		else
		{
			*max = theRangeOptions.endValue;
		}
	}
	else
	{
		*max = *maxChannel*(*step)+(*minTheta);
	}
}

void writeXFITFile(convertParamBlock *paramBlock)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	long *channelData = NULL;
	float *intensityData = NULL;
	int progressValue=0;
	char lineToOutput[300];
	double tTheta;
	double counts;
	calPoint *constStepCalPoints = NULL;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	double max, min;
	int error;
	
	MCAFileHeader MCAHeader;
	GUFIHeader GUFIFileHeader;
	
	constStepCalPoints = NULL;
	if (paramBlock->conversionType == kTWOTHETASPLINE); 
	{
		spline(calPoints, header.numOfCalibPoints, 0);
	}
	if (readFile(paramBlock, &channelData, &intensityData, &MCAHeader, &GUFIFileHeader) != 0)
	{
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints); 
			constStepCalPoints = NULL;
		}
		return;
	}
		if (paramBlock->inputFileType == kCHN)
	{
		size = MCAHeader.num_chans; 
	}
	else
	{
		size = GUFIFileHeader.numPoints;
	}
	if (constStepCalPoints!= NULL)
	{
		free(constStepCalPoints);
	}
	constStepCalPoints = malloc(size*sizeof(calPoint)); 
	if (constStepCalPoints==NULL)
	{
		MessagePopup("No Memory", "There isn't enough memory to do this!");
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	findAFreeFileName(paramBlock->outputFileName, "dat", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	setConstStepVars(paramBlock, channelData, intensityData, paramBlock->conversionType, constStepCalPoints, &MCAHeader, &GUFIFileHeader, paramBlock->progressPanelHandle);
	GetHeaderInfo(paramBlock, &step, &minTheta, &maxChannel, &min, &max, size);
	writeXFITHeader(min, step, max, outputFile);
	for (channel =0; channel<maxChannel; channel++)
	{
		progressValue = (int)((float)((float)channel/(float)maxChannel)*100); //work out the amount of the file which has been processed.
		if ((progressValue)%10==0) //if the value is divisible by 10 then...
		{
			SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
		}
		lineToOutput[0]=0x00;
		tTheta = minTheta + step * channel;
		if (paramBlock->inputFileType == kCHN)
		{
			error = getTThetaAndCountsCHN(channelData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &MCAHeader, constStepCalPoints);
		}
		else
		{
			error = getTThetaAndCountsGUFI(intensityData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &GUFIFileHeader, constStepCalPoints);
		}
		if (0 == error)
		{	
			if ((paramBlock->conversionOtherOptions & kFLOATINGCOUNTS) == 0)
			{
				sprintf(lineToOutput, "%8d", (int)counts);	
			}
			else
			{
				sprintf(lineToOutput, "%8.02f", counts);
			}
			WriteFile(outputFile, lineToOutput, strlen(lineToOutput));
			if ((channel+1)%10 == 0)
			{
				WriteFile(outputFile, "\n", 1);
			}
		}
	}
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	CloseFile(outputFile);
	if (constStepCalPoints != NULL)
	{
		free(constStepCalPoints);
		constStepCalPoints = NULL;
	}
	if (channelData != NULL)
	{
		free(channelData);
		channelData = NULL;
	}
}

void writeCPIFile(convertParamBlock *paramBlock)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	long *channelData = NULL;
	float *intensityData = NULL; 
	int progressValue=0;
	char lineToOutput[300];
	double tTheta;
	double counts;
	calPoint *constStepCalPoints = NULL;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	double min, max;
	int error;
	
	MCAFileHeader MCAHeader;
	GUFIHeader GUFIFileHeader;
	
	constStepCalPoints = NULL;
	if (paramBlock->conversionType == kTWOTHETASPLINE); 
	{
		spline(calPoints, header.numOfCalibPoints, 0);
	}
	if (readFile(paramBlock, &channelData, &intensityData, &MCAHeader, &GUFIFileHeader) != 0)
	{
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;
	}
	if (paramBlock->inputFileType == kCHN)
	{
		size = MCAHeader.num_chans; 
	}
	else
	{
		size = GUFIFileHeader.numPoints;
	}
	if (constStepCalPoints!= NULL)
	{
		free(constStepCalPoints);
	}
	constStepCalPoints = malloc(size*sizeof(calPoint)); 
	if (constStepCalPoints==NULL)
	{
		MessagePopup("No Memory", "There isn't enough memory to do this!");
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	findAFreeFileName(paramBlock->outputFileName, "cpi", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	setConstStepVars(paramBlock, channelData, intensityData, paramBlock->conversionType, constStepCalPoints, &MCAHeader, &GUFIFileHeader, paramBlock->progressPanelHandle);
	GetHeaderInfo(paramBlock, &step, &minTheta, &maxChannel, &min, &max, size);
	writeCPIHeader(min, step, max, outputFile);
	for (channel =0; channel<maxChannel; channel++)
	{
		progressValue = (int)((float)((float)channel/(float)maxChannel)*100); //work out the amount of the file which has been processed.
		if ((progressValue)%10==0) //if the value is divisible by 10 then...
		{
			SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
		}
		lineToOutput[0]=0x00;
		tTheta = minTheta + step * channel;
		if (paramBlock->inputFileType == kCHN)
		{
			error = getTThetaAndCountsCHN(channelData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &MCAHeader, constStepCalPoints);
		}
		else
		{
			error = getTThetaAndCountsGUFI(intensityData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &GUFIFileHeader, constStepCalPoints);
		}
		if (0 == error)
		{	
			if ((paramBlock->conversionOtherOptions & kFLOATINGCOUNTS) == 0)
			{
				sprintf(lineToOutput, "%d", (int)counts);	
			}
			else
			{
				sprintf(lineToOutput, "%8.04f", counts);
			}
			WriteLine(outputFile, lineToOutput, strlen(lineToOutput));
		}
	}
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	CloseFile(outputFile);
	if (constStepCalPoints != NULL)
	{
		free(constStepCalPoints);
		constStepCalPoints = NULL;
	}
	if (channelData != NULL)
	{
		free(channelData);
		channelData = NULL;
	}
}

void writeLHMPFile(convertParamBlock *paramBlock)
{
	char outputFileName[300];
	char message[300];
	int outputFile = 0;
	long channel;
	long *channelData = NULL;
	float *intensityData = NULL; 
	int progressValue=0;
	char lineToOutput[300];
	double tTheta;
	double counts;
	calPoint *constStepCalPoints = NULL;
	long size = 0;
	double step;
	double minTheta;
	long maxChannel;
	double min, max;
	int error;
	
	MCAFileHeader MCAHeader;
	GUFIHeader	GUFIFileHeader;
	
	constStepCalPoints = NULL;
	if (paramBlock->conversionType == kTWOTHETASPLINE); 
	{
		spline(calPoints, header.numOfCalibPoints, 0);
	}
	if (readFile(paramBlock, &channelData, &intensityData, &MCAHeader, &GUFIFileHeader) != 0)
	{
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		} 
		return;
	}
	if (paramBlock->inputFileType == kCHN)
	{
		size = MCAHeader.num_chans; 
	}
	else
	{
		size = GUFIFileHeader.numPoints;
	}
	if (constStepCalPoints!= NULL)
	{
		free(constStepCalPoints);
	}
	constStepCalPoints = malloc(size*sizeof(calPoint)); 
	if (constStepCalPoints==NULL)
	{
		MessagePopup("No Memory", "There isn't enough memory to do this!");
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	findAFreeFileName(paramBlock->outputFileName, "dat", outputFileName, 300);
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kOUTPUTFILENAME, outputFileName); 
	if ((outputFile = OpenFile(outputFileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII))<0)
	{
		sprintf(message, "Couldn't open output file: %s\nYour disk my be full, you may not have write permission or you may have the file already open.\nWill skip conversion.", paramBlock->inputFileName);
		MessagePopup("File Error", message);
		if (constStepCalPoints != NULL)
		{
			free(constStepCalPoints);
			constStepCalPoints = NULL;
		}
		if (channelData != NULL)
		{
			free(channelData);
			channelData = NULL;
		}
		return;	
	}
	setConstStepVars(paramBlock, channelData, intensityData, paramBlock->conversionType, constStepCalPoints, &MCAHeader, &GUFIFileHeader, paramBlock->progressPanelHandle);
	GetHeaderInfo(paramBlock, &step, &minTheta, &maxChannel, &min, &max, size);
	writeLHMPHeader(min, step, max, outputFile);
	for (channel =0; channel<maxChannel; channel++)
	{
		progressValue = (int)((float)((float)channel/(float)maxChannel)*100); //work out the amount of the file which has been processed.
		if ((progressValue)%10==0) //if the value is divisible by 10 then...
		{
			SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
		}
		lineToOutput[0]=0x00;
		tTheta = minTheta + step * channel;
		if (paramBlock->inputFileType == kCHN)
		{
			error = getTThetaAndCountsCHN(channelData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &MCAHeader, constStepCalPoints);
		}
		else
		{
			error = getTThetaAndCountsGUFI(intensityData, channel, &tTheta, &counts, paramBlock->conversionType, paramBlock->conversionOtherOptions, &GUFIFileHeader, constStepCalPoints);
		}
		if (0 == error)
		{	
			if ((paramBlock->conversionOtherOptions & kFLOATINGCOUNTS) == 0)
			{
				sprintf(lineToOutput, "%8d", (int)counts);	
			}
			else
			{
				sprintf(lineToOutput, "%8.02f", counts);
			}	
			WriteFile(outputFile, lineToOutput, strlen(lineToOutput));
			if ((channel+1)%10 == 0)
			{
				WriteFile(outputFile, "\n", 1);
			}
		}
	}
	SetCtrlVal(paramBlock->progressPanelHandle, kPROGRESS_kPROGRESSBAR, 100.0);
	CloseFile(outputFile);
	if (constStepCalPoints != NULL)
	{
		free(constStepCalPoints);
		constStepCalPoints = NULL;
	}
	if (channelData != NULL)
	{
		free(channelData);
		channelData = NULL;
	}
}
