/*****************************************************************************************/
/*	By Stefan Pantos. E-Mail Address: stefuser@yahoo.com								 */
/* Last Modified: 21/07/2000															 */
/*****************************************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
#include <formatio.h>
#include <ansi_c.h>
#include <utility.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "DiffConv.h"
#ifndef __OPTIONS_H__
#include "Options.h"
#endif
#include "DiffConvTypes.h"
#include "SplineOptionPanel.h"
#include "ConstantStepOptPanel.h"
#include "myWindowUtils.h"
#include "RangeOptPanel.h" 
#include "Conversion.h" 
#include "CPIOps.h"  

//******************Global Variables********************************
static int mainPanel;				//The main panel for the application
/******************************************************************/

/******************SetCalibrationMessage*************************/
/* This method sets the text messages on the main panel to the  */
/* current values that they have.								*/
/****************************************************************/
void SetCalibrationMessages()
{
	char theValue[30]; //A string for the message to be made in.

	sprintf(theValue, "%9.03f", header.K); 						//Create the string in theValue with the string representaition of the value of the firstChannelTwoTheta value.
	SetCtrlVal(mainPanel, MP_kCHANNELONEVALUE, theValue);				//Set the value of the first channel message to the string theValue.
	sprintf(theValue, "%9.03f", header.C);    						//Create the string in theValue with the string representaition of the value of the twoThetangleOerChannel value.
	SetCtrlVal(mainPanel, MP_kTWOTHETAVALUE, theValue);					//Set the value of the two theta message to the string in theValue.
}
//-----------------------------------------------------------------
/*****************displayOptionPanel*************/
/*   This displays the first option panel in the*/ 
/*  optionPanelFlags which is finds.			*/
/*	Parameters: 								*/
/*		optionPanelFlags: Each bit corraspons to*/
/*    a option panel. The first bit which is set*/
/*    to 1 makes it's corrasponding option panel*/
/*	  to be displayed.                          */
/************************************************/
void displayOptionPanels(int optionPanelFlags)
{
	panelStruct *panelInfo = NULL;  //The panel structure which is to be displayed.
	
	if ((optionPanelFlags & constantStepOptions) >0)//Is the constant step flag set in the optionsPanelFlag var if so then...
	{
		panelInfo = createPanelStruct(LoadPanel(mainPanel, "DiffConv.uir", kCONSPANEL), mainPanel, malloc(sizeof(int)));	//create the panel structure for the constant step option panel and load it in.
		initConstStepOpsPanel(panelInfo);			//Set up the constant step options panel for displaying.
	}
	else if ((optionPanelFlags & rangeOption) > 0)  //else is if the range option flag is set then...
	{
		panelInfo= createPanelStruct(LoadPanel(mainPanel, "DiffConv.uir", kRANGEOPT), mainPanel, malloc(sizeof(int))); 	//Create the panel structer for the range panel and load the panel into memory.
		initRangeOpsPanel(panelInfo);				//Set up the range panel for displaying.
	}
	else if ((optionPanelFlags & CPIOptions) > 0)	//Else if the cpi flag is set then...
	{
		panelInfo= createPanelStruct(LoadPanel(mainPanel, "DiffConv.uir", kCPIOPTS), mainPanel, malloc(sizeof(int)));		//Create the CPI option panel structure and load the panel into memory.
		initCPIOptPanel(panelInfo);					//Set up the CPI option panel up ready to be displayed.
	}
	else if ((optionPanelFlags & convertOption) > 0) //Else if the convert options is the only flag left then...
	{
		convertFiles(mainPanel);			// Convert all the files. 
		return;
	}
	else	//Else if there are no flags set...
	{
		DisplayPanel(mainPanel); //bring the main panel to the front   
		return;
	}
	*((int *)panelInfo->otherData) = optionPanelFlags;  //pass the panel which is just about to be displayed the options flags.
	InstallPopup(panelInfo->panelHandle);				//Display the current options panel as a Popup.
}

/****************************displayOutputFileType*****************/
/*  This looks to see what type of file is to be converted to and */
/* then displays the type on the main panel.					  */
/******************************************************************/
void displayOutputFileType()
{
	int value=0; 		//The value of the output file type ring.
	int dimmed;	 		//Wither the output file type ring is dimmed or not.

	GetCtrlVal(mainPanel, MP_kOUTPUTFILETYPE, &value); //Get the current output value type.
	switch (value)
	{
		case kTEXT:
			SetCtrlVal(mainPanel, MP_kOUTPUTFILETYPEVALUE, kTXTFILESTRING);	//Display that the output type will be a Text File.
		break;
		case kXFIT:
			SetCtrlVal(mainPanel, MP_kOUTPUTFILETYPEVALUE, kXFITFILESTRING);//Display that the output type will be a XFIT File. 
		break;
		case kCPI:
			SetCtrlVal(mainPanel, MP_kOUTPUTFILETYPEVALUE, kCPIFILESRING);  //Display that the output type will be a CPI File.
		break;
		case kLHMP:
			SetCtrlVal(mainPanel, MP_kOUTPUTFILETYPEVALUE, kLHPMFILESTRING);//Display that the output type will be a LHMP File. 
		break;
		case kGUFI:
			SetCtrlVal(mainPanel, MP_kOUTPUTFILETYPEVALUE, kGUFIFILESTRING);//Display that the output type will be a GUFI File. 
		break;
	}
	if (value == kTEXT) //If the output file type is a text file then... 
	{
		GetCtrlAttribute(mainPanel, MP_kOUTPUTFILETYPE, ATTR_DIMMED, &dimmed);  //see if the output file type ring is dimmed
		if (dimmed == 0)	//If the file output type ring isn't dimmed then...
		{
			SetCtrlAttribute(mainPanel, MP_kCONSTANTSTEP, ATTR_DIMMED, 0); //Highlight the constent set options panel.	
		}
	}
	else  //Else if it's going to output any other file type other than text then...
	{
		SetCtrlVal(mainPanel, MP_kCONSTANTSTEP, 1);						//This sets the value of the constent step option to one.
		SetCtrlAttribute(mainPanel, MP_kCONSTANTSTEP, ATTR_DIMMED, 1);	//Dim the the constent 
	}
}

/***********************dimAsNeeded****************************/
/*  This dims the buttons on the mainPanel as they are needed.*/
/**************************************************************/
void dimAsNeeded()
{
	int numOfItems;//To store the number of items in the list.
	int twoTheta; //This is to store whether the two theta check box is check or not.
	int linearCheck; //This is to store whether the linear convertion radio button has been checked;
	int fileType = kDONTKNOW;
	int radioVal; // The value of the last radio button which was looked at.
	
	GetNumListItems(mainPanel, MP_kFILES, &numOfItems);  //Get the number of items in the list.
	GetCtrlVal(mainPanel, MP_kCONVERT, &twoTheta);
	GetCtrlVal(mainPanel, MP_kLINEARCONVERTION, &linearCheck);       //Gets the current value of the linear convertion radio button
	if (numOfItems>0)	//If there are some items in the list 
	{
		SetCtrlAttribute(mainPanel, MP_kREMOVEFILE, ATTR_DIMMED, 0); 	//Undim the remove button
		SetCtrlAttribute(mainPanel, MP_kCONVERTBUTTON, ATTR_DIMMED, 0);	//Undim the convert button
		GetValueFromIndex(mainPanel, MP_kFILES, 0, &fileType);
	}
	else //if the aren't any items then
	{
		SetCtrlAttribute(mainPanel, MP_kREMOVEFILE, ATTR_DIMMED, 1); 	//Dim the Remove button
		SetCtrlAttribute(mainPanel, MP_kCONVERTBUTTON, ATTR_DIMMED, 1);	//Dim the convert button
	}
	if (fileType == kGUFI || fileType == kDONTKNOW) //If the input file type is GUFI then...
	{
		SetCtrlVal(mainPanel, MP_kCONVERT, 1);							//Set the output 2-theta file check box to on.
		SetCtrlAttribute(mainPanel, MP_kCONVERT, ATTR_DIMMED, 1);		//Dim the Output 2-theta file check box.
		SetCtrlAttribute(mainPanel, MP_kNULLCONVERSION, ATTR_DIMMED, 0);//Highlight the NULL conversion radio button.
	}  
	else if(fileType == kCHN) //Else if the file isn't a Gufi file then...
	{
		GetCtrlVal(mainPanel, MP_kNULLCONVERSION, &radioVal);	//Get the value of the NULL conversion radio button.
		if (radioVal == 1)		//IF the radio button is on then...
		{
			SetCtrlVal(mainPanel, MP_kLINEARCONVERTION, 1);		//Set the linear conversion radio button to on.
			SetCtrlVal(mainPanel, MP_kNULLCONVERSION, 0);		//Set the NULL covnersion radio button to off
		}
		SetCtrlAttribute(mainPanel, MP_kNULLCONVERSION, ATTR_DIMMED, 1);  //Dim the NULL conversion radio button.
		SetCtrlAttribute(mainPanel, MP_kCONVERT, ATTR_DIMMED, 0);		  //Highlight the Output 2-theta check box.
	}
	GetCtrlVal(mainPanel, MP_kCONVERT, &twoTheta);  //Get the value of the two theta check box.
	if (twoTheta==1)    //If the check box is check then...
	{
		SetCtrlAttribute(mainPanel, MP_kLINEARCONVERTION, ATTR_DIMMED, 0); //Highlight the linear convertion radio button 
		SetCtrlAttribute(mainPanel, MP_kSPLINECONVERTION, ATTR_DIMMED, 0); //Highlight thespline convertion radio button.
		SetCtrlAttribute(mainPanel, MP_kCONVERTOPTS, ATTR_DIMMED, 0);	   //Highlight the  convertion decortion box.
		SetCtrlAttribute(mainPanel, MP_kROUNDTOINT, ATTR_DIMMED, 0);	   //Highlight the  round to int check box.
		SetCtrlAttribute(mainPanel, MP_kOUTPUTFILETYPE, ATTR_DIMMED, 0);   //Highlight the output file type ring.
		SetCtrlAttribute(mainPanel, MP_kLIMMITRANGECB, ATTR_DIMMED, 0);	   //Highlight the limit range check box.
		if (linearCheck == 1)	//If both the two theta check box and the linear convertion radio button are checked then .. 
		{
			SetCtrlAttribute(mainPanel, MP_kCHANNELONEVALUE, ATTR_DIMMED, 0); //Dim the channel value number
			SetCtrlAttribute(mainPanel, MP_CHANONEANGLELABEL, ATTR_DIMMED, 0);//Dim the channel value label
			SetCtrlAttribute(mainPanel, MP_TWOTHETALABEL, ATTR_DIMMED, 0);	  //Dim the two theta value label
			SetCtrlAttribute(mainPanel, MP_kTWOTHETAVALUE, ATTR_DIMMED, 0);   //Dim the two theta value number
		}
		else	//If the output 2-theta check box is on and the linear radio button isn't then...
		{
			SetCtrlAttribute(mainPanel, MP_kCHANNELONEVALUE, ATTR_DIMMED, 1);  //Undim the channel value number 
			SetCtrlAttribute(mainPanel, MP_CHANONEANGLELABEL, ATTR_DIMMED, 1); //Undim the channel value label
			SetCtrlAttribute(mainPanel, MP_TWOTHETALABEL, ATTR_DIMMED, 1);	   //Undim the two theta value label
			SetCtrlAttribute(mainPanel, MP_kTWOTHETAVALUE, ATTR_DIMMED, 1);	   //Undim the two theta value number 
		}
		displayOutputFileType();										   //Displays what file type is going to be output.
	}
	else  //If the check box isn't check then..
	{
		SetCtrlAttribute(mainPanel, MP_kLINEARCONVERTION, ATTR_DIMMED, 1);  //Dim the linear convertion radio button  
		SetCtrlAttribute(mainPanel, MP_kSPLINECONVERTION, ATTR_DIMMED,1);   //Dim the spline convertion radio button.
		SetCtrlAttribute(mainPanel, MP_kCONVERTOPTS, ATTR_DIMMED, 1);		//Dim the  convertion decortion box.
		SetCtrlAttribute(mainPanel, MP_kROUNDTOINT, ATTR_DIMMED, 1);	   //Dim the  round to int check box.
		SetCtrlAttribute(mainPanel, MP_kCONSTANTSTEP, ATTR_DIMMED, 1);	   //Dim the  constant step check box.
		SetCtrlAttribute(mainPanel, MP_kLIMMITRANGECB, ATTR_DIMMED, 1);
		SetCtrlAttribute(mainPanel, MP_kNULLCONVERSION, ATTR_DIMMED, 1);   //Dim the NULL conversion Radio Button.
		SetCtrlAttribute(mainPanel, MP_kCHANNELONEVALUE, ATTR_DIMMED, 1);  //dim the channel value number 
		SetCtrlAttribute(mainPanel, MP_CHANONEANGLELABEL, ATTR_DIMMED, 1); //dim the channel value label
		SetCtrlAttribute(mainPanel, MP_TWOTHETALABEL, ATTR_DIMMED, 1);	   //dim the two theta value label
		SetCtrlAttribute(mainPanel, MP_kTWOTHETAVALUE, ATTR_DIMMED, 1);	   //dim the two theta value number 
	}
}

/**********************doesFileListContain****************************/
/* This checks to see if the string theString is in the list		 */
/* Parameters:														 */
/*		theString - the string which you want to check is in the list*/
/* Returns: Returns true if the string is in the list.				 */
/*********************************************************************/
int doesFileListContain(char *theString)							 
{
	int index; //The index of the current list item i'm up to.
	int sizeOfList; //the number of items in the list
	int result=0; //value which should be out put from the method
	char label[350]; //the path returned from the list.
	
	GetNumListItems(mainPanel, MP_kFILES, &sizeOfList);  //Get the number of items in the list
	for (index=0; index<sizeOfList; index++) //While there are still items to check
	{
		GetLabelFromIndex(mainPanel, MP_kFILES, index, label); //get the lable on the list
		result |= !strcmp(theString, label); //See the path in the list is the same as the path past in to the method
	}
	return result;  //Return the result
}

/************************************main**************************/
/*  This is the main method which is called when the program is   */
/*	first started.												  */
/******************************************************************/
int main (int argc, char *argv[])
{
	int index; //Used to run through the command line arguments.
	
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	GetProjectDir(projectDir);
	if ((mainPanel = LoadPanel (0, "DiffConv.uir", MP)) < 0) //Loads the main panel into memory
		return -1;
	getCalibration();			//Reads in the current calibration numbers into memory from the defualt file
	SetCalibrationMessages();	//Displays the calibration number on the main panel
	DisplayPanel (mainPanel);	//Displays the main panel
	if (argc>1)	//If there where more than one parameters passed then...
	{
		for (index=1; index<argc; index++) //While the are no my parameters to check.
		{
			InsertListItem(mainPanel, MP_kFILES, -1, argv[index], 0); //Add the parameters to the file.
		}
	}
	RunUserInterface ();		//Runs the interface
	return 0;
}

/**********************lowercase********************/
/*	This is a procedure which makes the char passed*/
/*	an lower chase letter.						   */
/*  I know that there is probably already a method */
/*	which does this but I couldn't find it.		   */
/*	Parameters:									   */
/*		theChar: the char which you want to be made*/
/*		into an upper case letter.				   */
/*	Returns:									   */
/*		The lower case version on the letter passed*/
/*		in theChar								   */
/***************************************************/
char lowercase(char theChar)  
{
	if (theChar <= 90 && theChar >= 65) //If the letter is between or equal to 'A' and 'Z'.
	{
		return theChar+32; //make the letter lower case.
	}
	return theChar; //Return the letter as it was passed if it is already lower case.
}

/******************strCmp**************************/
/*  This compares two string and returns if they  */
/*	are equal or not.							  */
/*	Parameters:									  */
/*			string1: A pointer to one of the 	  */
/*			string to compare.					  */	
/*			string2: A pointer to the other string*/
/*			to compare with.					  */
/*	Return:										  */
/*			If the strings string1 and string2 	  */
/*			point to are the same then it returns */
/*			a 0 otherwise it returns a 1. 		  */
/**************************************************/
int strCmp(char *string1, char *string2)
{
	int length = strlen(string1); //The length of string1.
	int i;						  //Which char is it up to.
	char char1;					  //The lower case of the current char.
	char char2;					  //The lower case of the current char.
	
	if (length==strlen(string2))  //If the strings are the same length then...
	{
		for (i=0; i<length; i++)  //Go through all the chars in the strings.
		{
			char1 = lowercase(string1[i]); //get the lower case version of the current char in string1.
			char2 = lowercase(string2[i]); //get the lower case version of the current char in string2.
		 	if (char1!=char2)			   //If the two chars don't equel each other then...
		 	{
		 		return 1;				  //Return that the two strings don't equal each other.
		 	}
		}
		return 0;						//If it gets out of the for loop then there are equal and so it returns a 0.
	}
	return 1;							//If there are not the same length then they aren't equal.
}

/******************getFileType******************/
/*	This gets returns the file type of the file*/
/*	passed to it.							   */
/*	Parameters:								   */
/*			fileName: A poitner to the path of */
/*		the file which you want the type of.   */
/*  Returns:								   */
/*			return a number which corrasponds  */
/*			to a file type constants.		   */
/*			kGUFI - GUFI file *.guf			   */
/*			kCHN - CHN file *.mca			   */
/*			kDONTKNOW - Unknown file type	   */
/***********************************************/
int getFileType(char *fileName)
{
	int index = strlen(fileName);   //The current position in the path name. Starting from the end.
	
	while (fileName[index]!='.' && index-4<index)	//While the current char isn't a '.' and it hasn't moved more than 4 chars from the end. 
	{
		index--;				//Move to the previose char.
	}
	if (fileName[index] == '.') //If the last char looked at was a . then...
	{
		if (strCmp(fileName+index+1, "chn") == 0)	//See if the file extension is equal to chn and if it is then...
		{
			return kCHN;						    //return that the file type is a kCHN file.
		}
		else if (strCmp(fileName+index+1, "guf") == 0)  //else see if the file extension is equal to guf then... 
		{
			return kGUFI;							//Return that the file type is a kGUFI file.
		}
	}
	return kDONTKNOW;								//Other wise return that the file type isn't know.
}

/**************Callbacks for the main Panel********************************************/
/*******************addFileButton***********************************/
/*  This is the callback for the add button. It shows a multiSelect*/
/* window and then adds each one to the list as long as it's not   */
/* already in it												   */
/*******************************************************************/
int CVICALLBACK addFilesButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int numOfFiles; //The number of files needing to be added.
	int clickedOK;  //Whether the user clicked ok in the multifileselectpopup
	int index;      //The current index the loop is up to.
	char **fileNames =NULL; // A pointer to the pointer of the file names.
	int listType;	
	int numInList;
	int fileType;
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetNumListItems(panel, MP_kFILES, &numInList);
			if (numInList>0)
			{
				GetValueFromIndex(panel, MP_kFILES, 0, &listType); 				
			}
			clickedOK=MultiFileSelectPopup(projectDir, "*.chn", "*.chn; *.guf", "Select the input files", 0, 0, 1, &numOfFiles, &fileNames); //Show the multifileselect.
			if (clickedOK) //See if the OK button was clicked.
			{              //If it is then...
				for (index=0; index<numOfFiles; index++)//Go through all the files returned by the MulitFileSelect.
				{
					if (!doesFileListContain(fileNames[index])) //Check that the file isn't already in the list.
					{  //If it doesn't then...
						fileType = getFileType(fileNames[index]);   //Gets the file type of the file which is to be added to the list.
						if (fileType == listType || numInList == 0) //if the file which is going to be added is the same as the type of the list of the list is empty then...
						{
							InsertListItem(mainPanel, MP_kFILES, -1, fileNames[index], getFileType(fileNames[index])); //Insert the file into the list.
						}
					}
				}
			}
			break;
		}
	dimAsNeeded();//Check to see if any of the controls need dimming or undimming.
	return 0;
}

/*************************removeFileButton***********************/
/*  This remove the currently selected item in the list from the*/ 
/* list.													    */
/****************************************************************/
int CVICALLBACK removeFilesButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int selectedItem; //The index of the currently selected item.
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlIndex(mainPanel, MP_kFILES, &selectedItem);	  //Get the index of the currently selected item
			DeleteListItem(mainPanel, MP_kFILES, selectedItem, 1);//Remove that time from the list.
			break;
		}
	dimAsNeeded();  //See if any controls need to be dimmed.
	return 0;
}
//-----------------------------------------------------------------
//-------------Convert Button--------------------------------------
//  This calls all the needed functions needed converting the files.
// And hides the mainPanel;
int CVICALLBACK convertButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int convertOptions = convertOption;
	int controlVal=0;
	
	switch (event)
		{
		case EVENT_COMMIT:
			HidePanel(panel);	// Hide the mian panel.
			GetCtrlVal(panel, MP_kCONVERT, &controlVal);
			if (controlVal == 1)
			{
				GetCtrlVal(panel, MP_kCONSTANTSTEP, &controlVal);
				if (controlVal==1)
				{
					convertOptions |= constantStepOptions;
				}
				GetCtrlVal(panel, MP_kLIMMITRANGECB, &controlVal);
				if (controlVal == 1)
				{
					convertOptions |= rangeOption; 
				}
				
			}
			GetCtrlAttribute(mainPanel, MP_kOUTPUTFILETYPE, ATTR_DIMMED, &controlVal);
			if (controlVal == 0)
			{
				GetCtrlVal(panel, MP_kOUTPUTFILETYPE, &controlVal);
				if (controlVal == kCPI)
				{
					convertOptions |= CPIOptions;
				}
			}
			displayOptionPanels(convertOptions);
			break;
		}
	return 0;
}
//-----------------------------------------------------------------
//------------loadOptions------------------------------------------
//	This is called when the load options menu item is clicked on
// It loads a pre file in for the options.
void CVICALLBACK loadOptions (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	char optionFileName[300]; //Name of the option file
	
	if ( FileSelectPopup (projectDir, "*.pre", "*.pre", "Select Option File",
			   VAL_OK_BUTTON, 0, 1, 1, 0, optionFileName)) //Display the file select popup for the user to select which one they whant to open
	{
		readOptionsFile(optionFileName); //Read in the file and sets the configeration values to the values in the file.
	}
	SetCalibrationMessages();
}
//-----------------------------------------------------------------
//----------saveOptions--------------------------------------------
//  This is called when the save option menu item is selected from 
// the menu bar. It displays the FileSelectPopup. When the user has 
// selected a file name or clicked Cancel then the file is saved.
// It repeats this until the file is saved to the file or the Cancel
// button on the FileSelectPopup is clicked.
void CVICALLBACK saveOptions (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	char optionFileName[300];  //Name of the option file.
	int saved, clickedOK; //True if the file was saved.
	
	do
	{
		clickedOK=FileSelectPopup (projectDir, "*.pre", "*.pre", "Select Option File",
				   VAL_OK_BUTTON, 0, 1, 1, 0, optionFileName);	//Display the File Select Window for the user to select the name of the file.
		if (clickedOK)//Check to see if the user clicked the Ok button in the file select window.
		{	//If they did then...
		
			saved=saveOptionsFile(optionFileName, 0); // Save the file and if there is a file with that name already then ask if they wish to replace it.
		}
	}
	while (!saved && clickedOK); //If the file didn't save but they did click OK on the file select window do it again.
}
//-----------------------------------------------------------------
//-----------Quit Menu---------------------------------------------
//  This is called when the quit menu is selected. It then quits 
// the whole program
void CVICALLBACK quitMenu (int menuBar, int menuItem, void *callbackData,
		int panel)
{		  
	QuitUserInterface(0);  //Quit the program.
}

/*********************tThetaConversionCB**************************/
/* This is called when the two-theta check box is clicked on. It */
/* Dims the radio buttons as appropreat to the value of the check*/ 
/* box															 */
/*****************************************************************/
int CVICALLBACK tConvertFileCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int twoTheta;
	
	switch (event)
		{
		case EVENT_COMMIT:
				dimAsNeeded(); 
			break;
		}
	return 0;
}

//----------linearRadioButton-------------------------------------
// This is called when the linear conversion radio button is clicked
// on. It sets the spline radio button to the opersit value and dims
// the linear reg coefficents as appropreat.
int CVICALLBACK linearRadioButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int radioVal;
	
	switch (event)
		{
		case EVENT_COMMIT:
				GetCtrlVal(mainPanel, MP_kLINEARCONVERTION, &radioVal); //Get the value of the Linear convertion radio button.
				if (radioVal==0)
				{
					SetCtrlVal(mainPanel, MP_kSPLINECONVERTION, 1); //Set the value of the Spline convertion radio button to the opposit value to the linea convertion radio button.
				}
				else
				{
					SetCtrlVal(mainPanel, MP_kNULLCONVERSION, 0); //Set the NULL convertion radio button to the opposit value to which the spline convertion radio button is set to.
					SetCtrlVal(mainPanel, MP_kSPLINECONVERTION, 0); //Set the value of the Spline convertion radio button to the opposit value to the linea convertion radio button.
				}
			break;
		}
	dimAsNeeded(); //Check to see what needs to be dimmed and dim it if it needs to be. 
	return 0;
}

//---------splineRadioButton--------------------------------------
// This is called when the spline radio button is clicked on. It 
// sets the linear radio button to the opersit value and dims the
// linear reg coeficents as appropreat.
int CVICALLBACK splineRadioButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int radioVal;
	
	switch (event)
		{
		case EVENT_COMMIT:
				GetCtrlVal(mainPanel, MP_kSPLINECONVERTION, &radioVal); //Get the value of the spline convertion radio button
				if (radioVal==0)
				{
					GetCtrlAttribute(mainPanel, MP_kNULLCONVERSION, ATTR_DIMMED, &radioVal);
					if (radioVal == 1)
					{
						SetCtrlVal(mainPanel, MP_kLINEARCONVERTION, 1); //Set the value of the Spline convertion radio button to the opposit value to the linea convertion radio button.
					}
					else
					{
						SetCtrlVal(mainPanel, MP_kNULLCONVERSION, 1); //Set the NULL convertion radio button to the opposit value to which the spline convertion radio button is set to.
					}
				}
				else
				{
					SetCtrlVal(mainPanel, MP_kNULLCONVERSION, 0); //Set the NULL convertion radio button to the opposit value to which the spline convertion radio button is set to.
					SetCtrlVal(mainPanel, MP_kLINEARCONVERTION, 0); //Set the value of the Spline convertion radio button to the opposit value to the linea convertion radio button.
				}
		}
	dimAsNeeded();  //Check to see what needs to be dimmed and dim it if it needs to be.
	return 0;
}

int CVICALLBACK nullConversionRB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int radioVal;
	
	switch (event)
		{
		case EVENT_COMMIT:
				GetCtrlVal(mainPanel, MP_kNULLCONVERSION, &radioVal); //Get the value of the NULL conversion radio button
				if (radioVal==0)
				{
					SetCtrlVal(mainPanel, MP_kLINEARCONVERTION, 1); //Set the value of the Spline convertion radio button to the opposit value to the linea convertion radio button.
				}
				else
				{
					SetCtrlVal(mainPanel, MP_kLINEARCONVERTION, 0); //Set the NULL convertion radio button to the opposit value to which the spline convertion radio button is set to.
					SetCtrlVal(mainPanel, MP_kSPLINECONVERTION, 0); //Set the value of the Spline convertion radio button to the opposit value to the linea convertion radio button.
				}
			break;
		}
	dimAsNeeded(); //Check to see what needs to be dimmed and dim it if it needs to be.
	return 0;
}

//-----------------------------------------------------------------
//---------------optionsMenu---------------------------------------
//  This loads the option panel from file then creates a panel 
// structer to be past to it. It then displays the panel as a popup
// window.
void CVICALLBACK optionsMenu (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	panelStruct *optionPanel = NULL; //Panel structure pointer to be past to callbacks.
	
	optionPanel = createPanelStruct(LoadPanel(mainPanel, "DiffConv.uir", kLINOPTION), mainPanel, NULL);
	if (optionPanel->panelHandle<0) //Load the panel and check that it was loaded.
	{
		MessagePopup("Panel Error", "Bugger! There was an error while loading the options panel"); //Error message is the panel isn't loaded.
		return;
	}
	SetCtrlAttribute(optionPanel->panelHandle, kLINOPTION_kCANCELBUTTON, ATTR_CALLBACK_DATA, optionPanel);  //Set the opton panels cancel button's callback to the panels panelstruct.
	SetCtrlAttribute(optionPanel->panelHandle, kLINOPTION_kOKBUTTON, ATTR_CALLBACK_DATA, optionPanel);		 //Set the opton panels OK button's callback to the panels panelstruct.
	SetPanelAttribute(optionPanel->panelHandle, ATTR_CALLBACK_DATA, optionPanel);		 //Set the option panels callback data to it's own structure so that it can remove it's self from memory.
	SetCtrlVal(optionPanel->panelHandle, kLINOPTION_kANGLECHANNELONE, header.K); //Sets the angle of channel one numeric to the current value of the Calibration options
	SetCtrlVal(optionPanel->panelHandle, kLINOPTION_NUMERIC, header.C);			   //Sets the 2-theta angle per channel numeric to the current value of the Calibration options
	SetMenuBarAttribute(GetPanelMenuBar(optionPanel->panelHandle), OPTIONMENU_kDEFAULT_kSAVEDEFAULT, ATTR_CALLBACK_DATA, optionPanel); //Set the option panels SaveDefault menu items callback to the panels panelstruct.
	InstallPopup(optionPanel->panelHandle);  //It displays the option panel as a popup.
}
//-----------------------------------------------------------------
//-------------mainPanelMethod-------------------------------------
// This called when ever an event happens to the main panel. 
// When the main panel comes into focus then the it checks to see if
// its just come back from one it's windows. If it has then the 
// callbackData pointer will not be NULL. If this happens then it 
// persums that the pointer points to a panel structure. Removes the
// panel from memory then the panel structure. It then sets the main
// panels callback attribute to NULL.  
int CVICALLBACK mainPanelMethod (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:
			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
				if (calPoints!=NULL)  //Frees any memory which has been allocated for the calibartion points.
				{
					free(calPoints);
					calPoints = NULL;
					header.numOfCalibPoints;
				}
				QuitUserInterface(0);  //Quit the program.
			break;
		}
	return 0;
}
//-----------------------------------------------------------------

void CVICALLBACK SplineOptionsMenu (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	panelStruct *splineOptionPanel = NULL;
	int index;
	
	splineOptionPanel = createPanelStruct(LoadPanel(mainPanel,"DiffConv.uir", kSPLINEOPT), mainPanel, malloc(sizeof(xyPoints)*kMAXNUMOFPOINTS));
	if (splineOptionPanel->panelHandle < 0) //Load the spline option panel into memory. If there is an error while doing this then...
	{
		MessagePopup("Error", "Couldn't load Auto Calculate panel"); //Tell the user that there was an error wil loading the panel.
		if (((int)splineOptionPanel->otherData)>0)
		{
			free(splineOptionPanel);
		}
		return;  //Leave the method.
	}
	if (((int)splineOptionPanel->otherData)<=0)
	{
		DiscardPanel(splineOptionPanel->panelHandle);
		MessagePopup("Error", "You have run out of memory."); //Tell the user that there isn't enough memory.
		return;
	}
	for (index=0; index<kMAXNUMOFPOINTS; index++) //Loop through all the Array Elements.
	{
		((xyPoints *)(splineOptionPanel->otherData))[index].deleted=1; //Make all the elements in the xyPoints array empty. 
	}
	splineOptionPanel->parentPanelHandle=mainPanel;   //Set the panel structures parent handle to the main panel handle.
	SetCtrlAttribute(splineOptionPanel->panelHandle, kSPLINEOPT_OKBUTTON, ATTR_CALLBACK_DATA, splineOptionPanel);      //Pass the control it's panels structure.
	SetCtrlAttribute(splineOptionPanel->panelHandle, kSPLINEOPT_kCANCELBUTTON, ATTR_CALLBACK_DATA, splineOptionPanel); //Pass the control it's panels structure.
	SetCtrlAttribute(splineOptionPanel->panelHandle, kSPLINEOPT_kREMOVEBUTTON, ATTR_CALLBACK_DATA, splineOptionPanel); //Pass the control it's panels structure.
	SetCtrlAttribute(splineOptionPanel->panelHandle, kSPLINEOPT_kADDBUTTON, ATTR_CALLBACK_DATA, splineOptionPanel);	   //Pass the control it's panels structure.
	SetCtrlAttribute(splineOptionPanel->panelHandle, kSPLINEOPT_kREMOVEALL, ATTR_CALLBACK_DATA, splineOptionPanel);	   //Pass the control it's panels structure.
	SetCtrlAttribute(splineOptionPanel->panelHandle, kSPLINEOPT_kREMOVEALL, ATTR_CALLBACK_DATA, splineOptionPanel);	
	SetPanelAttribute(splineOptionPanel->panelHandle, ATTR_CALLBACK_DATA, splineOptionPanel);	
	SetMenuBarAttribute(GetPanelMenuBar(splineOptionPanel->panelHandle), OPTIONMENU_kDEFAULT_kSAVEDEFAULT, ATTR_CALLBACK_DATA, splineOptionPanel); //Set the option panels SaveDefault menu items callback to the panels panelstruct.
	SetMenuBarAttribute(GetPanelMenuBar(splineOptionPanel->panelHandle), kSPLINEMEN_kIMPORT_kIMPORTXFIT, ATTR_CALLBACK_DATA, splineOptionPanel); //Set the option panels Load Xfit menu items callback to the panels panelstruct.
	addPointsToList(splineOptionPanel->panelHandle, (xyPoints *)(splineOptionPanel->otherData));  //Add all the current points to the list box and to the xyPoints array.
	paramsDimAsNeeded(splineOptionPanel->panelHandle); //Dim and highlight the controls as they should be.
	InstallPopup(splineOptionPanel->panelHandle);	   //Display the spline panel.
}



int CVICALLBACK quitButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
				if (calPoints!=NULL)  //Frees any memory which has been allocated for the calibartion points.
				{
					free(calPoints);
					calPoints = NULL;
					header.numOfCalibPoints;
				}
				QuitUserInterface(0);  //Quit the program.          
			break;
		}
	return 0;
}

void CVICALLBACK constStepOption (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	displayOptionPanels(constantStepOptions);
}

void CVICALLBACK rangeOptions (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	displayOptionPanels(rangeOption);
}

int CVICALLBACK outputFileTypeMenu (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
				displayOutputFileType();
			break;
		}
	return 0;
}
