/**********************************************************************/
/*   By Stefan Pantos. E-Mail: Stefuser@yahoo.com                     */
/*	 Last Modified: 21/07/2000										  */
/**********************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
#include <userint.h>
#include <ansi_c.h>
#ifndef __OPTIONS_H__
#include "Options.h"
#endif
#include "DiffConvTypes.h"
#include "DiffConv.h"
#include "myWindowUtils.h"
#include "LinearOptionsPoints.h" 

/******************************************************************/
/*************Callbacks for the option Panel***********************/
//----------saveDeflaultMenu---------------------------------------
//  This is called when the save as default menu item in the option
// panel is clicked on. It first constructs the file name. Then 
// copies the values which are in the numerics into the Calibration
// options. Then saves these options in the Default file.
void CVICALLBACK saveDefaultMenu (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	char optionFileName[300];
	
	if (0==ConfirmPopup("Warning", "This will over write the spline option in the default file.\nDo you still whant to do this?"))
	{
		return;
	}
	sprintf(optionFileName, "%s\\Default.pre", projectDir); //Make the path name to the default option file 
	GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kLINOPTION_kANGLECHANNELONE, &header.K); //Get the changed values of the options from the numeric controls.
	GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kLINOPTION_NUMERIC, &header.C);
	saveOptionsFile(optionFileName, 1); //Save the files to the default file writting over the old ones.
}
//----------------------------------------------------------------- 

//------------optionOkButton---------------------------------------
//  This is called when the option panels OK button is clicked.
// It puts th values of the numeric controls into the callibration 
// varibale. It then sets it's parentPanels Callback to the the 
// option panels panel structure. It then remove the option panel 
// from the screen.
int CVICALLBACK optionOkButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kLINOPTION_kANGLECHANNELONE, &header.K); //Set the calibration varibales to the values in the option panel numerics.
			GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kLINOPTION_NUMERIC, &header.C);
			RemovePopup(((panelStruct *)callbackData)->panelHandle);
			windowToClose((panelStruct *) callbackData);
			SetCalibrationMessages();  //Set the main panels display of the configeration options to there new values.
			break;
		}
	return 0;
}
//----------------------------------------------------------------- 
//------------canncelButton----------------------------------------
//  This is called when the option panels cancel button is clicked
// It sets the option panels parent panels callback to the 
// options panels panel structure. It hen remove the Option panel 
// from the screen.
int CVICALLBACK cancelButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			RemovePopup(((panelStruct *)callbackData)->panelHandle); //Removes the option panel from the screen.
			windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}
//----------------------------------------------------------------- 

int CVICALLBACK linearOpPanel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
			RemovePopup(((panelStruct *)callbackData)->panelHandle); //Removes the option panel from the screen.
			windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}
