/**********************************************************************/
/*   By Stefan Pantos. E-Mail: Stefuser@yahoo.com                     */
/*	 Last Modified: 21/07/2000										  */
/**********************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
#include <analysis.h>
#include <ansi_c.h>
#include <userint.h>
#include "DiffConv.h"
#include "LinearOptionsPoints.h"
#include "myWindowUtils.h"
#include "Options.h" 

void *oldMenuFunction = NULL;

void lineOptDimAsNeeded(int panelHandle);
void LinReg(double *xPoints, double *yPoints, int numOfPoints, double *slop, double *intercept);

void addPoint(panelStruct *panelInfo, double xPoint, double yPoint)
{
	int i = 0;
	char outputLine[200];
	
	while (((xyPoints *)(panelInfo->otherData))[i].deleted == 0)
	{
		i ++;
	}
	if (i<kMAXNUMOFPOINTS)
	{
		((xyPoints *)(panelInfo->otherData))[i].tThetaValue = yPoint;
		((xyPoints *)(panelInfo->otherData))[i].channelNum = xPoint;
		((xyPoints *)(panelInfo->otherData))[i].deleted = 0;
		sprintf(outputLine, "%8.04f  -> %8.04f", yPoint, xPoint); 
		InsertListItem(panelInfo->panelHandle, kLINEOPT_kDATAPOINTLIST, -1, outputLine, i);  //Add the line to the list box and set it's value to the index of the number in the array.
	}
	lineOptDimAsNeeded(panelInfo->panelHandle); //Dim and highlight any controls which need to be because of any change with the controls.
}

void initLinearOptionsPanel(panelStruct *panelInfo)
{
	int i, menuBar;
	
	if (panelInfo->otherData != NULL)
	{
		free(panelInfo->otherData);
		panelInfo->otherData = NULL;
	}
	panelInfo->otherData = malloc(kMAXNUMOFPOINTS*sizeof(xyPoints));
	for (i = 0; i<kMAXNUMOFPOINTS; i++)
	{
		((xyPoints *)(panelInfo->otherData))[i].deleted = 0x01;	
	}
	if (calPoints != NULL)
	{
		for (i=0; i<header.numOfCalibPoints; i++)
		{
			addPoint(panelInfo, calPoints[i].xPoint, calPoints[i].yPoint);
		}
	}
	SetCtrlAttribute(panelInfo->panelHandle, kLINEOPT_kREMOVEBUTTON, ATTR_CALLBACK_DATA, panelInfo);
	SetCtrlAttribute(panelInfo->panelHandle, kLINEOPT_kREMOVEALL, ATTR_CALLBACK_DATA, panelInfo); 
	SetCtrlAttribute(panelInfo->panelHandle, kLINEOPT_kOKBUTTON, ATTR_CALLBACK_DATA, panelInfo); 
	SetCtrlAttribute(panelInfo->panelHandle, kLINEOPT_kADDBUTTON, ATTR_CALLBACK_DATA, panelInfo); 
	SetCtrlAttribute(panelInfo->panelHandle, kLINEOPT_kCANCELBUTTON, ATTR_CALLBACK_DATA, panelInfo); 
	SetPanelAttribute(panelInfo->panelHandle, ATTR_CALLBACK_DATA, panelInfo);
	menuBar = GetPanelMenuBar(panelInfo->panelHandle);
	GetMenuBarAttribute(menuBar, kSPLINEMEN_kDEFAULTMENU_kSAVEASD, ATTR_CALLBACK_FUNCTION_POINTER, &oldMenuFunction);
	SetMenuBarAttribute(menuBar, kSPLINEMEN_kDEFAULTMENU_kSAVEASD, ATTR_CALLBACK_FUNCTION_POINTER, (void *)saveAsDefaultMenuLin);
	SetMenuBarAttribute(menuBar, kSPLINEMEN_kDEFAULTMENU_kSAVEASD, ATTR_CALLBACK_DATA, panelInfo);
	SetMenuBarAttribute(menuBar, kSPLINEMEN_kIMPORT_kIMPORTXFIT, ATTR_CALLBACK_DATA, panelInfo); //Set the option panels Load Xfit menu items callback to the panels panelstruct.
}

void lineOptDimAsNeeded(int panelHandle)
{
	int numOfPoints; 
	
	GetNumListItems(panelHandle, kLINEOPT_kDATAPOINTLIST, &numOfPoints);  //Get the number of items in the list control.
	if (numOfPoints<2)
	{
		SetCtrlAttribute(panelHandle, kLINEOPT_kOKBUTTON, ATTR_DIMMED, 1);
		if (numOfPoints == 0)
		{
			SetCtrlAttribute(panelHandle, kLINEOPT_kREMOVEBUTTON, ATTR_DIMMED, 1);
			SetCtrlAttribute(panelHandle, kLINEOPT_kREMOVEALL, ATTR_DIMMED, 1);
		}
		else
		{
			SetCtrlAttribute(panelHandle, kLINEOPT_kREMOVEBUTTON, ATTR_DIMMED, 0);
			SetCtrlAttribute(panelHandle, kLINEOPT_kREMOVEALL, ATTR_DIMMED, 0);
		}
	}
	else
	{
		SetCtrlAttribute(panelHandle, kLINEOPT_kOKBUTTON, ATTR_DIMMED, 0);
		SetCtrlAttribute(panelHandle, kLINEOPT_kADDBUTTON, ATTR_DIMMED, 0);
		SetCtrlAttribute(panelHandle, kLINEOPT_kREMOVEBUTTON, ATTR_DIMMED, 0);
		SetCtrlAttribute(panelHandle, kLINEOPT_kREMOVEALL, ATTR_DIMMED, 0);
	}
	if (numOfPoints == kMAXNUMOFPOINTS-1)
	{
		SetCtrlAttribute(panelHandle, kLINEOPT_kADDBUTTON, ATTR_DIMMED, 1);
	}
}

void removePoint(panelStruct *panelInfo, int index)
{
	int itemVal;
	
	GetValueFromIndex(panelInfo->panelHandle, kLINEOPT_kDATAPOINTLIST, index, &itemVal);//Gets the value of the line for removing the numbers from the array.
	((xyPoints *)panelInfo->otherData)[itemVal].deleted = 1;
	DeleteListItem(panelInfo->panelHandle, kLINEOPT_kDATAPOINTLIST, index, 1);		//This deletes the selected line in the list box from the list box.
	lineOptDimAsNeeded(panelInfo->panelHandle); //Dim and highlight any controls which need to be because of any change with the controls.
}

int CVICALLBACK addToDataListLin (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	double tTheta, channelNum;
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kLINEOPT_kTWOTHETAVALUE, &tTheta);
			GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kLINEOPT_kCHANNELNUMBER, &channelNum);
			addPoint(((panelStruct *)callbackData), channelNum, tTheta);
			break;
		}
	return 0;
}

int CVICALLBACK cancelSplineOpPanelLin (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int menuBar;
	
	switch (event)
		{
		case EVENT_COMMIT:
				menuBar = GetPanelMenuBar(panel);
				SetMenuBarAttribute(menuBar, kSPLINEMEN_kDEFAULTMENU_kSAVEASD, ATTR_CALLBACK_FUNCTION_POINTER, oldMenuFunction);
				RemovePopup(((panelStruct*)callbackData)->panelHandle);
				windowToClose((panelStruct*)callbackData);
			break;
		}
	return 0;
}
 
void calcCoefficients(panelStruct *panelInfo)
{
	int i, indexOfPoints=0;
	double *xPoints = NULL, *yPoints= NULL;
	xyPoints *points = panelInfo->otherData;
	
	GetNumListItems(panelInfo->panelHandle, kLINEOPT_kDATAPOINTLIST, &header.numOfCalibPoints);
	if (calPoints !=NULL)
	{
		free(calPoints);
		calPoints = NULL;
	}
	calPoints = malloc(sizeof(calPoint)*header.numOfCalibPoints);
	if (xPoints !=NULL)
	{
		free(xPoints);
		xPoints = NULL;
	}
	xPoints = malloc(sizeof(double)*header.numOfCalibPoints);
	if (yPoints !=NULL)
	{
		free(yPoints);
		yPoints = NULL;
	}
	yPoints = malloc(sizeof(double)*header.numOfCalibPoints);  
	for (i = 0; i<header.numOfCalibPoints; i++)
	{
		GetValueFromIndex(panelInfo->panelHandle, kLINEOPT_kDATAPOINTLIST, i, &indexOfPoints);//Gets the value of the line for removing the numbers from the array.
		if (points[i].deleted == 0)
		{
			calPoints[i].yPoint = points[indexOfPoints].tThetaValue;
			calPoints[i].xPoint = points[indexOfPoints].channelNum;
			yPoints[i] = points[indexOfPoints].tThetaValue;
			xPoints[i] = points[indexOfPoints].channelNum;
		}
	}
	LinReg(xPoints, yPoints, header.numOfCalibPoints, &header.C, &header.K); 
	free(xPoints);
	free(yPoints);
}

int CVICALLBACK okOpPanelLin (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int menuBar;
	
	switch (event)
		{
		case EVENT_COMMIT:
			menuBar = GetPanelMenuBar(panel);
			calcCoefficients((panelStruct*)callbackData);
			SetMenuBarAttribute(menuBar, kSPLINEMEN_kDEFAULTMENU_kSAVEASD, ATTR_CALLBACK_FUNCTION_POINTER, oldMenuFunction);
			RemovePopup(((panelStruct*)callbackData)->panelHandle);
			SetCalibrationMessages();  //Set the main panels display of the configeration options to there new values.
			windowToClose((panelStruct*)callbackData);
			break;
		}
	return 0;
}


void LinReg(double *xPoints, double *yPoints, int numOfPoints, double *slop, double *intercept)
{
	double xMean, yMean, xStDev, yStDev, r;
	int i;
	
	xMean = yMean =0;
	for (i =0; i< numOfPoints; i++)
	{
		xMean += xPoints[i]; 
		yMean += yPoints[i];
	}
	xMean /= numOfPoints;
	yMean /= numOfPoints;
	xStDev = yStDev = 0;
	for (i= 0; i< numOfPoints; i++)
	{
		xStDev += sqr((xPoints[i]-xMean));
		yStDev += sqr((yPoints[i]-yMean));
	}
	xStDev /= (numOfPoints-1);
	yStDev /= (numOfPoints-1);
	xStDev = sqrt(xStDev);
	yStDev = sqrt(xStDev);
	r = 0;
	for (i=0; i<numOfPoints; i++)
	{
		r += ((xPoints[i]-xMean)/xStDev)*((yPoints[i]-yMean)/yStDev)	;
	}
	r /= (numOfPoints - 1);
	*slop = r * (yStDev/xStDev);
	*intercept = yMean - (*slop) * xMean;
}

int CVICALLBACK removeAllLin (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int numOfPoints = 0, i;
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetNumListItems(((panelStruct *)callbackData)->panelHandle, kLINEOPT_kDATAPOINTLIST, &numOfPoints);  //Get the number of items in the list control.
			for (i = 0; i<numOfPoints; i++)
			{
				removePoint(((panelStruct *)callbackData), 0);
			}
			break;
		}
	return 0;
}

int CVICALLBACK removeFromDataListLin (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int index;
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlIndex(((panelStruct *)callbackData)->panelHandle, kLINEOPT_kDATAPOINTLIST, &index);			//Get the index of the line which is currently selected in the list box control.
			removePoint(((panelStruct *)callbackData), index);
			break;
		}
	return 0;
}

int CVICALLBACK linearOptionPointsMethod (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{	 
	int menuBar;
	
	switch (event)
		{
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
				menuBar = GetPanelMenuBar(panel);
				SetMenuBarAttribute(menuBar, kSPLINEMEN_kDEFAULTMENU_kSAVEASD, ATTR_CALLBACK_FUNCTION_POINTER, oldMenuFunction);
				RemovePopup(((panelStruct*)callbackData)->panelHandle);
				windowToClose((panelStruct*)callbackData);
			break;
		}
	return 0;
}

void CVICALLBACK linearCalibration (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	panelStruct *panelInfo = createPanelStruct(LoadPanel(panel, "DiffConv.uir", kLINEOPT), panel, NULL);
	initLinearOptionsPanel(panelInfo);
	InstallPopup(panelInfo->panelHandle);
}

void CVICALLBACK saveAsDefaultMenuLin (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	char optionFileName[300];
	
	calcCoefficients(callbackData);
	sprintf(optionFileName, "%s\\Default.pre", projectDir); //Make the path name to the default option file 
	saveOptionsFile(optionFileName, 1); //Save the files to the default file writting over the old ones.
}
