/**********************************************************************/
/*   By Stefan Pantos. E-Mail: Stefuser@yahoo.com                     */
/*	 Last Modified: 21/07/2000										  */
/**********************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
#include "DiffConv.h"
#include <utility.h>
#include <userint.h>
#include "Options.h"
#include "Standards.h"
#include <formatio.h>
#include "myWindowUtils.h"

#define kHEADERLABEL "#Header"
#define kCALIBRASIONDATALABEL "#Calibrasion Data"
#define kNUMOFDATAPOINTS "numOfDataPoints"
#define kSTRINGLENGTH 300

void writeOptionFile(char *fileName, Calibration *header);
void readOptionFile(int fileHandle, Calibration *header);
void processDataLine(calPoint **dataStorage, char *lineToProcess, int *numOfPoints, int fileHandle);

calPoint *calPoints=NULL;

Calibration header;

char projectDir[300]; //The directory where the program is stored.

//-----getCalibration---------------------------------------------------
// This read the deflault calibration values from the default option 
// file. If one doesn't exist then it creates one.
void getCalibration(void)
{
	int optionFile; // the handle to the option file
	char optionFileName[300]; // Then name of the option file
	
	sprintf(optionFileName, "%s\\Default.pre", projectDir); //Create the path to the default option file
	//Set initalise the Calibration variables.
	header.version=1;
	header.K=10.939501;
	header.C=0.02692846;
	header.numOfCalibPoints = 4;
	if (calPoints != NULL)
	{
		free(calPoints);
		calPoints = NULL;
	}
	calPoints= malloc(sizeof(calPoint)*4);
	calPoints[0].yPoint=28.783;
	calPoints[0].xPoint=655; 
	calPoints[1].yPoint=47.304;
	calPoints[1].xPoint=1353;
	calPoints[2].yPoint=56.122;
	calPoints[2].xPoint=1681;
	calPoints[3].yPoint=69.132;
	calPoints[3].xPoint=2167;
	DisableBreakOnLibraryErrors();
	optionFile = OpenFile(optionFileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII); //Open the default option file just as read only.
	EnableBreakOnLibraryErrors();
	if (optionFile>=0)  //Test to see it the file opened
	{//If it did then...
		readOptionFile(optionFile, &header); //read the data from the file	 
		CloseFile(optionFile); //Closes the file
	}
	else	//If the file didn't open then a new file needs to be made.
	{
		saveOptionsFile(optionFileName, 1);//Creates a new file and saves the normal values into it.
	}
}

//----------------------------------------------------------------------
//-----readOptionFile---------------------------------------------------
// This reads the options from a file with the path which is past in 
// from fileName.
// Parameters:
//		fileName - The path of the file which is to be read from
void readOptionsFile(char fileName[])
{
	int optionFile; //A handle to the option file
	
	DisableBreakOnLibraryErrors();
	optionFile = OpenFile(fileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII);  //open the option file
	EnableBreakOnLibraryErrors();
	if (optionFile>=0) // if the file opened
	{
		readOptionFile(optionFile, &header); //read the data from the file	
		CloseFile(optionFile); //close the file
		return;
	}
	//Else if the file didn't open then
	MessagePopup("File Error", "Couldn't open option file."); // display a message saying that there was an error
}	 
//----------------------------------------------------------------------

void processLine(char *string, int *foundLabel, int bytesRead, Calibration *header, int fileHandle)
{
	char tempString[kSTRINGLENGTH];
	int index;
	
	if (string[0]=='#')
	{
		if (strcmp(string, kHEADERLABEL) == 0)
		{
			*foundLabel = 1;
		}
		else if (strcmp(string, kCALIBRASIONDATALABEL) == 0)
		{
			*foundLabel = 2;
		}
	}
	else if (*foundLabel == 1)
	{
		index = 0;
		while (index < strlen(string) && string[index]!='=' && string[index]!=0x00)
		{
			tempString[index] = string[index];
			index++;
		}
		tempString[index] = 0x00;
		if (strcmp(tempString, "C") == 0)
		{
			header->C = atof(string+index+1);
		}
		else if (strcmp(tempString, "K") == 0)
		{
			header->K = atof(string+index+1);
		}
	}
	else if (*foundLabel == 2)
	{
		processDataLine(&calPoints, string, &header->numOfCalibPoints, fileHandle);
	}
}

void getTheNumbers(double *number1, double *number2, char *line)
{
	char firstNum[kSTRINGLENGTH];
	char delimeter = '\t';
	int index = 0; 
	
	while (line[index]!='\t' && index<kSTRINGLENGTH-1)
	{
		index++;
	}
	if (index >= kSTRINGLENGTH-1)
	{
		delimeter =' ';
	}
	index = nextWord(line, firstNum, delimeter, 0, kSTRINGLENGTH);
	*number1 = atof(firstNum);
	index = nextWord(line, firstNum, delimeter, 0, kSTRINGLENGTH); 
	*number2 = atof(line+index);
}

void processDataLine(calPoint **dataStorage, char *lineToProcess, int *numOfPoints, int fileHandle)
{
	int index, bytesRead=1, numOfPointsRead = 0;
	char tempString[kSTRINGLENGTH];
	
	if (lineToProcess[0]<'0' || lineToProcess[0]>'9')
	{
		index = 0;
		while (lineToProcess[index]!=0x00 && lineToProcess[index] != '=' && index < kSTRINGLENGTH-1)
		{
			tempString[index] = lineToProcess[index];
			index ++;
		}
		tempString[index] = 0x00;
		if (strcmp(tempString, kNUMOFDATAPOINTS) == 0)
		{
			*numOfPoints = atoi(lineToProcess+index+1);
			if (*dataStorage != NULL)
			{
				free(*dataStorage);
				*dataStorage = NULL;
			}
			*dataStorage = malloc(sizeof(calPoint)* (*numOfPoints));
			for (index= 0; index < *numOfPoints && bytesRead>=0; index ++)
			{
				bytesRead = readLine(fileHandle, tempString, kSTRINGLENGTH);
				if (bytesRead > 0)
				{
					numOfPointsRead ++;
					getTheNumbers(&(*dataStorage)[index].xPoint, &(*dataStorage)[index].yPoint, tempString); 
				}
			}
			if (*numOfPoints != numOfPointsRead)
			{
				*numOfPoints = numOfPointsRead;
				MessagePopup("Option File Error", "There was a small error with the options file.\nI have loaded all the points I could");
			}
		}
	}
}

long readLine(int fileHandle, char *string, int maxLength)
{
	long bytesRead = 0, index = -1;
	
	do
	{
		index++;
		bytesRead = ReadFile(fileHandle, string+index, 1);	
	}
	while (bytesRead>0 && string[index]!='\n');
	string[index]=0x00;
	if (bytesRead > 0)
	{
		return index;
	}
	else
	{
		return -1;
	}
}

void readOptionFile(int fileHandle, Calibration *header) //void readOptionFileHeader(int fileHandle, calHeader *header)
{
	char string[kSTRINGLENGTH];
	long bytesRead;
	int foundLabel = 0;
	
	do
	{
		bytesRead = readLine(fileHandle, string, kSTRINGLENGTH);
		if (bytesRead > 0)
		{
			processLine(string, &foundLabel, bytesRead, header, fileHandle);
		}
	}
	while (bytesRead>=0);
	
}

//--------saveOptionsFile-----------------------------------------------
//  This saves the option file to a path which is defined by fileName.
// If the file already exists then a replace window comes up to ask the
// user if they wish to replace it.
// Parameters:
//      fileName - the path of the file to be save to.
//      noReplaceOption - if there is a file with the path the same as 
//         the one stored in fileName then if it should ask to write over
//         it or to write over it any way.
// Returns: 1 if it wrote to the file and 0 if it didn't.
int saveOptionsFile(char fileName[], int noReplaceOption)
{
	int optionFile; // A handle to the option file to write to.
	int replace=1;  // The result from the replace box.
	int stringIndex=0; //The current persition in a string.
	char confirmBoxMessage[300];//The message to be shown on the replace popup.
	
	DisableBreakOnLibraryErrors();
	optionFile=OpenFile(fileName, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII); //Try's to see if the file is there already
	EnableBreakOnLibraryErrors();                                         //F
	if (optionFile>=0 && !noReplaceOption) 								  //If the file is there and it should ask whether to over write
	{ // then...
		CloseFile(optionFile);	//closes the file.
		optionFile=0; 			//Makes sure that the file isn't closed again when it is already closed.
		//These two while loops find the last / in the file.
		while (fileName[stringIndex]!=0)	//While the pointer isn't pointing to the end of the list.
		{
			stringIndex++; 					//Move to the next char in the string
		}
		while (fileName[stringIndex]!='\\') //While the pointer doesn't point to a back slash
		{
			stringIndex--; 					//Move to the prev char in the string.
		}
		sprintf(confirmBoxMessage, "Do you want to replace the file:%s", fileName+stringIndex); //Put the first string together witht the second string from the last backslash in it.
		replace = ConfirmPopup("Replace", confirmBoxMessage); //Show the ConfirmPopup to ask the user if they want to over write the file.
		if (!replace) 		//See what they said.
		{ 					//If they said no then...
			return 0; 		//return the value 0 and leave the method
		}
	}
	if (optionFile>0) //See if the optionFile is open.
	{//If it is then...
		CloseFile(optionFile); //Close the file.
	}
	writeOptionFile(fileName, &header); 			//Write the calibration values to the file.
	return 1; 				//Return 1 to say that it did write to the file.
}
//---------------------------------------------------------------------- 


void writeOptionFile(char *fileName, Calibration *header)
{
	char line[300];
	int index;
	int optionFile;
	
	optionFile = OpenFile(fileName, VAL_WRITE_ONLY, VAL_TRUNCATE, VAL_ASCII); //Open the default option file just as read only.
	WriteLine(optionFile, "#y=Cx+K\n#Header", 15);
	sprintf(line, "K=\t%8.06f", header->K);
	WriteLine(optionFile, line, strlen(line));
	sprintf(line, "C=\t%8.06f", header->C);
	WriteLine(optionFile, line, strlen(line)); 
	WriteLine(optionFile, "", 0);
	if (header->numOfCalibPoints>0)
	{
		WriteLine(optionFile, "#Observed Channel for the peaks, calculated 2-theta value for peaks", 67);
		WriteLine(optionFile, kCALIBRASIONDATALABEL, strlen(kCALIBRASIONDATALABEL));
		sprintf(line, "%s= %d", kNUMOFDATAPOINTS, header->numOfCalibPoints);
		WriteLine(optionFile, line, strlen(line));
		for (index =0; index < header->numOfCalibPoints; index ++)
		{
			sprintf(line, "%8.04f\t%8.04f", calPoints[index].xPoint, calPoints[index].yPoint);
			WriteLine(optionFile, line, strlen(line));
		}
		WriteLine(optionFile, "", 0);
	}
	CloseFile(optionFile);
}
