#include <userint.h>
#include "DiffConv.h"
#include "SplineOptionPanel.h"
#include "myWindowUtils.h"
/****************************************************************************/
/*   By Stefan Pantos. E-Mail: Stefuser@yahoo.com                     		*/
/*	 Last Modified: 21/07/2000										  		*/
/*  These methods all realate to spline fitting and the spline option panel.*/
/****************************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
/***************************Swap*********************************/
/*  This is to swap to calPoint values. This is used by 		*/
/* sortCalPoints.												*/
/* Parameters:													*/
/* 			point1: This is a pointer one of the calPoints to be*/
/*        swaped.												*/
/*			point2: This is a pointer to a point which is to be */
/*        swaped with point1.									*/
/****************************************************************/
void swap(calPoint *point1, calPoint *point2)
{
	calPoint tempPoint;
	
	//Put the values of point1 into a temporary store
	tempPoint.xPoint=point1->xPoint; 
	tempPoint.yPoint=point1->yPoint;
	tempPoint.yDashed = point1->yDashed;
	//Put the values of point2 into point1's location.
	point1->xPoint=point2->xPoint;
	point1->yPoint=point2->yPoint;
	point1->yDashed=point2->yDashed;
	//Put the saved values of point1 into point2's location.
	point2->xPoint=tempPoint.xPoint;
	point2->yPoint=tempPoint.yPoint;
	point2->yDashed=tempPoint.yDashed;
}

/********************sortCalPoints*******************************/
/* This is to sort the calibration points into assending order. */
/* Feel free to put a better sort methored here it's just I 	*/
/* couldn't be bothered writing a better one.					*/
/* Parameters:													*/
/*			  points: is a pointer to a list of calPoints which */
/*			need sorting.										*/
/*            numberOfPoints: is the number of calPoints which  */
/*          points, points to.									*/
/****************************************************************/
void sortCalPoints(calPoint *points, int numOfPoints)
{
	int pos1=0, pos2=1;
	
	//Compare each item in the list with the others.
	for (pos1=0; pos1<numOfPoints-1; pos1++)
	{
		for (pos2=pos1+1; pos2<numOfPoints; pos2++)
		{
			if (points[pos1].xPoint>points[pos2].xPoint)  //If the points poitned to by pos is less than the point pointed to by pos2 then...
			{
				swap(&(points[pos1]), &(points[pos2]));   //Swap the values of the two points
			}
		}
	}
}


/************************sqr**********************/
/*  This just calculaits the square of the number*/
/*	past.										 */
/*	Parameter:									 */
/*		The number to be sqrared				 */
/*	Returns:									 */
/*		The result of num*num					 */
/*************************************************/
double sqr(double num)
{
	return num*num;
}

/**************************Spline Methods**************************/
/*  This Method Calulates the the spline curve for all the points */
/* passed in dataPoints. 
/******************************************************************/
void spline(calPoint *dataPoints, int n, int progressPanel, int linEdges)
{
	int i, k;
	double p, qn, sig, un, *u = NULL;
	double ypn = 0;
	double yp1 = 0;
	int totalLoops = 2*n-3, loopUpTo = 0;
	double progressValue;
	double sumOfXPoints, sumOfYPoints;
	double xMean, yMean;
	double xSd, ySd;
	double r = 0;
	
	if (linEdges)
	{
		for (i=0; i < n; i++)
		{
			sumOfXPoints +=dataPoints[i].xPoint;
			sumOfYPoints +=dataPoints[i].yPoint;
		}
		xMean = sumOfXPoints / n;
		yMean = sumOfYPoints / n;
		sumOfXPoints = 0;
		sumOfYPoints = 0;
		for (i=0; i < n; i++)
		{
			sumOfXPoints += sqr(dataPoints[i].xPoint - xMean);
			sumOfYPoints += sqr(dataPoints[i].yPoint - yMean);
		}
		xSd = sqrt(sumOfXPoints/(n-1));
		ySd = sqrt(sumOfYPoints/(n-1)); 
		for (i=0; i < n; i++)
		{
			r += (dataPoints[i].xPoint*dataPoints[i].yPoint - xMean *yMean);
		}
		r = (r/(n-1))/(xSd*ySd);
		ypn = yp1 = r *(ySd/xSd);
	}
	SetWaitCursor(1);
	if (progressPanel != 0)
	{
		SetCtrlVal(progressPanel, kPROGRESS_kPROGRESSBAR, 0.0);
		SetCtrlAttribute(progressPanel, kPROGRESS_kPROGRESSBAR, ATTR_LABEL_TEXT, "Calculating Spline");
	}
	sortCalPoints(dataPoints, n);
	if (u!= NULL)
	{
		free(u);
		u = NULL;
	}
	u=malloc(sizeof(double) *(n-1));
	if (yp1 > 0.99e30)
	{
		dataPoints[0].yDashed = u[0] = 0.0;
	}
	else
	{
	   dataPoints[0].yDashed = -0.5;
	   u[0] = (3.0/(dataPoints[1].xPoint-dataPoints[0].xPoint))*((dataPoints[1].yPoint-dataPoints[0].yPoint)/(dataPoints[1].xPoint-dataPoints[0].xPoint)-yp1);
	}
	for (i=1;i<n-1;i++)
	{
		sig = (dataPoints[i].xPoint-dataPoints[i-1].xPoint)/(dataPoints[i+1].xPoint-dataPoints[i-1].xPoint);
		p = sig*dataPoints[i-1].yDashed+2.0;
		dataPoints[i].yDashed =  (sig-1.0)/p;
		u[i] = (dataPoints[i+1].yPoint-dataPoints[i].yPoint)/(dataPoints[i+1].xPoint-dataPoints[i].xPoint)-(dataPoints[i].yPoint-dataPoints[i-1].yPoint)/(dataPoints[i].xPoint-dataPoints[i-1].xPoint);
		u[i] = (6.0*u[i]/(dataPoints[i+1].xPoint-dataPoints[i-1].xPoint)-sig*u[i-1])/p;
		loopUpTo++;
		if (progressPanel!=0)
		{
			progressValue = ((float)loopUpTo/(float)totalLoops)*100;
			if (((int)progressValue)%5==0) //if the value is divisible by 10 then...
			{
				SetCtrlVal(progressPanel, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
			}
		}
	}
	if (ypn>0.99e30)
		qn=un=0.0;
	else
	{
		qn = 0.5;
		un = (3.0/(dataPoints[n-1].xPoint-dataPoints[i-2].xPoint))*(ypn-(dataPoints[n-1].yPoint-dataPoints[n-2].yPoint)/(dataPoints[n-1].xPoint-dataPoints[n-2].xPoint));
	}
	dataPoints[n-1].yDashed=(un-qn*u[n-2])/(qn*dataPoints[n-2].yDashed+1.0);
	for (k=n-2; k>=0; k--)
	{
		dataPoints[k].yDashed=dataPoints[k].yDashed*dataPoints[k+1].yDashed+u[k];
		loopUpTo++;
		progressValue = (((float)loopUpTo/(float)totalLoops)*100);
		if (progressPanel!=0)
		{
			progressValue = (int)((float)(loopUpTo/totalLoops)*100);
			if (((int)progressValue)%5==0) //if the value is divisible by 10 then...
			{
				SetCtrlVal(progressPanel, kPROGRESS_kPROGRESSBAR, (double)progressValue); //Update the value of the progress bar on the progress window.
			}
		}
	}
	free(u);
	if (progressPanel != 0)
	{
		SetCtrlAttribute(progressPanel, kPROGRESS_kPROGRESSBAR, ATTR_LABEL_TEXT, "Progress");
		SetCtrlVal(progressPanel, kPROGRESS_kPROGRESSBAR, 0.0);
	}
	SetWaitCursor(0);
}

/****************************splint**********************/
/* This method is to get the corrasponding y value for	*/ 
/* the x value passed to it.							*/
/* Parameters:											*/
/*      	 dataPoints: This is a pointer to a list of */
/*      calibaration points. The Splin method should	*/
/*      have been run on these points before being past */
/*      to this method.									*/
/*     	  	 n: This is the number of points which are  */
/* 		at the pointer dataPoints.						*/
/*			 x: This is the x value which you what the  */
/* 		corrasponding y value for.						*/
/*	Return: This returns the corrasponding y value for  */
/*       the x value which was passed.					*/
/********************************************************/
double splint(calPoint *dataPoints, int n, double x)
{
	int klo, khi, k;
	double h, b, a;
	
	klo = 0;
	khi = n-1;
	while (khi-klo > 1)
	{
		k = (khi+klo) >> 1;
		if (dataPoints[k].xPoint>x)
			khi = k;
		else
			klo = k;
	}
	h = dataPoints[khi].xPoint - dataPoints[klo].xPoint;
	if (h == 0.0)
	{
		MessagePopup("Error", "There was an error with one of you points!");
	}
	a = (dataPoints[khi].xPoint - x)/h;
	b = (x- dataPoints[klo].xPoint)/h;
	return a*dataPoints[klo].yPoint+b*dataPoints[khi].yPoint+((a*a*a-a)*dataPoints[klo].yDashed+(b*b*b-b)*dataPoints[khi].yDashed)*(h*h)/6.0;
}

/************************cleanUpAfterAutoCalcPanel************/
/*  This method is for getting ride of any allocated memory  */
/* which was allocated for use in this window.				 */
/* Parameters:												 */
/* 			pointsToPoints: This is a pointer to the memory  */
/*   	allocated for the points array.						 */
/*************************************************************/
void cleanUpAfterSplineOpPanel(xyPoints *pointerToPoints)
{
	free(pointerToPoints);
}

/********************addNumsToArray***************************/
/*  This is to add a point which has been entered from the 	 */
/* spline option panel into the array theArray where there is*/
/* a deleted element. 										 */
/* Parameters:												 */
/*			tThetaValue: The 2-theta value read from the     */
/*		panel which is to be entered into the array.		 */
/*			channelNum: The channel number read from the 	 */
/*		panel which needs to be etnered into the array.		 */
/*			theArray: this is the array which the values are */
/*		to be added to.										 */
/*	return: It returs the array index of where the points 	 */
/*         where put. If it Couldn't find an empty element   */
/*        then it returns -1.								 */
/*************************************************************/
int addNumsToArray(double tThetaValue, double channelNum, xyPoints *theArray)
{
	int index=0; //The current array ellement it's up to.
	
	while (index<kMAXNUMOFPOINTS && theArray[index].deleted==0)//Go through the array till it finds a slot in the array which has it's deleted flag set to 1 or it has check all the elements in the array.
	{
		index++;
	}
	if (index==kMAXNUMOFPOINTS && theArray[index].deleted==0) //If it didn't find a deleted element then...
	{
		return -1;    //Return the value -1.
	}
	theArray[index].tThetaValue=tThetaValue;   //Enter the 2-theta value into the free element.
	theArray[index].channelNum=channelNum;     //Enter the channel value into the free element.
	theArray[index].deleted=0;				   //Set the deleted flag to 0.
	return index;							   //Return the value of the index which has been entered.
}

/****************removeNumsFromArray*************************/
/* This delete the element in theArray at the index poited  */
/* to by index.												*/
/*	Parameters:												*/
/*			index: This is the index of the array element 	*/
/*		which you want to remove.							*/
/*          theArray: This is the array which you wish to 	*/
/*      remove the element from.							*/
/************************************************************/
void removeNumsFromArray(int index, xyPoints *theArray)
{
	theArray[index].deleted=1; //Set the elements deleted flag to 1.	
}

/************************paramsDimAsNeeded*******************/
/*  This dims the controls of the sline option panel as they*/
/*  should be for the current state of the other other      */
/*	controls. 												*/
/*  Parameters:												*/
/*			panelHandle: This is the handle of the spline 	*/
/*       option panel.										*/
/************************************************************/
void paramsDimAsNeeded(int panelHandle)
{
	int numOfListItems=0;  //For storing the number of items in the list control.
	int menuBar = GetPanelMenuBar(panelHandle);  //The handle of the menu bar for the spline option panel.
	
	GetNumListItems(panelHandle, kSPLINEOPT_kDATAPOINTLIST, &numOfListItems);  //Get the number of items in the list control.
	if (numOfListItems==0)	//If there aren't any items in the list then...
	{
		SetCtrlAttribute(panelHandle, kSPLINEOPT_kREMOVEBUTTON, ATTR_DIMMED, 1); //Dim the remove button so that you cannot try and remove items which aren't there.
		SetCtrlAttribute(panelHandle, kSPLINEOPT_kREMOVEALL, ATTR_DIMMED, 1); //Dim the remove all button so that you cannot try and remove items which aren't there.
	}
	else //Else if there are items in the list then..
	{
		SetCtrlAttribute(panelHandle, kSPLINEOPT_kREMOVEBUTTON, ATTR_DIMMED, 0);  //Highlight the control so that the user can use it.
		SetCtrlAttribute(panelHandle, kSPLINEOPT_kREMOVEALL, ATTR_DIMMED, 0); //Highlight the control so that the user can use it.
	}
	if (numOfListItems<3) //If there are less then three data points in the list then...
	{
		SetCtrlAttribute(panelHandle, kSPLINEOPT_OKBUTTON, ATTR_DIMMED, 1); // Dim the OK button so that youhave to enter more that thee points.
		SetMenuBarAttribute(menuBar, OPTIONMENU_kDEFAULT, ATTR_DIMMED, 1);  // Dim the save as deflaut menus so that it isn't possible to enter the save the settings with less than three point entered.
	}
	else //If there are more than three points then..
	{
		SetCtrlAttribute(panelHandle, kSPLINEOPT_OKBUTTON, ATTR_DIMMED, 0);  //Highlight the OK button.
		SetMenuBarAttribute(menuBar, OPTIONMENU_kDEFAULT, ATTR_DIMMED, 0);   //Highlight the Default menu item.
	}
	if (numOfListItems>=kMAXNUMOFPOINTS) //If there are the maximum number of points possible to enter then
	{
		SetCtrlAttribute(panelHandle, kSPLINEOPT_kADDBUTTON, ATTR_DIMMED, 1); //Dim the add button so that it isn't possible to enter any more.
	}																					
	else
	{
		SetCtrlAttribute(panelHandle, kSPLINEOPT_kADDBUTTON, ATTR_DIMMED, 0);  //Highloght the Add button/
	}
}

/***********************cancelAutoCalc***********************/
/*  This is called when the cancel button on the Spline 	*/
/* option panel is clicked.									*/
/************************************************************/
int CVICALLBACK cancelSplineOpPanel (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
				cleanUpAfterSplineOpPanel(((xyPoints *)((panelStruct *)callbackData)->otherData)); //Free the allocated memory for the xyPoint Array
				RemovePopup(((panelStruct*)callbackData)->panelHandle);	//Remove the panel from the screen.
				windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}

/***********************addToDataList************************/
/*  This is called when the add button in the Spline option */
/* panel is clicked on.										*/
/************************************************************/
int CVICALLBACK addToDataList (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	double tThetaValue; //The value read from the 2-theta numeric.
	double channelNum;	//The value read from the channel numeric.
	char listItemLine[30]; //The line which is to be displayed by the list Box.
	int arrayIndex;		//The index of the values in the array.
	
	switch (event)
		{
		case EVENT_COMMIT:
				GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kTWOTHETAVALUE, &tThetaValue);  //Get the 2-theta value from the 2-theta numeric.
				GetCtrlVal(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kCHANNELNUMBER, &channelNum);	  //Get the channel value from the channel number numeric.
				SetCtrlVal(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kTWOTHETAVALUE, 0.0000);		  //Set the value of the 2-theta numeric to 0.
				SetCtrlVal(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kCHANNELNUMBER, 0.0000);		  //Set the value of the channel number numeric to 0.
				sprintf(listItemLine, "%8.04f  -> %8.04f", tThetaValue, channelNum);						//Create the line which is to be displayed by the list box.
				arrayIndex=addNumsToArray(tThetaValue, channelNum, (xyPoints *)(((panelStruct *)callbackData)->otherData));	//Add the values read to the array and get the index of where in the array they where put.
				if (arrayIndex!=-1)	//If there was a space in the array to put the values then...
				{
					InsertListItem(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kDATAPOINTLIST, -1, listItemLine, arrayIndex);  //Add the line to the list box and set it's value to the index of the number in the array.
				}
				paramsDimAsNeeded(((panelStruct*)callbackData)->panelHandle); //Dim and highlight any controls which need to be because of any change with the controls.
			break;
		}
	return 0;
}

/*******************removeFromDataList***********************/
/*  This is called when the remove button in the spline 	*/
/* option panel is clicked on. 								*/
/************************************************************/
int CVICALLBACK removeFromDataList (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int value; //The value of the line which is to be removed from the list box.
	int index; //The index of the line which is to be removed from the list box.
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlIndex(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kDATAPOINTLIST, &index);			//Get the index of the line which is currently selected in the list box control.
			GetValueFromIndex(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kDATAPOINTLIST, index, &value);//Gets the value of the line for removing the numbers from the array.
			removeNumsFromArray(value, (xyPoints *)(((panelStruct *)callbackData)->otherData));						//This remove the numbers which are assocaited with the seleced list box line from the array.
			DeleteListItem(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kDATAPOINTLIST, index, 1);		//This deletes the selected line in the list box from the list box.
			paramsDimAsNeeded(((panelStruct*)callbackData)->panelHandle);											//This dims and highlights any controls on the spline option panel which needs to be.
			break;
		}
	return 0;
}

/************************removeAll******************************/
/*  This ia called when the remove all button is clicked on. It*/
/* remove all the items in the list box and sets all the       */
/* values of the array to deleted.							   */
/***************************************************************/
int CVICALLBACK removeAll (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i = 0;
	
	switch (event)
		{
		case EVENT_COMMIT:
				ClearListCtrl(((panelStruct *)callbackData)->panelHandle, kSPLINEOPT_kDATAPOINTLIST);  //Remove all the points from the list.
				for (i=0; i<kMAXNUMOFPOINTS; i++)  //Go through all the points in the array.
				{
					((xyPoints *)(((panelStruct *)callbackData)->otherData))[i].deleted=1; //Set it so that it is deleted.
				}
				paramsDimAsNeeded(((panelStruct *)callbackData)->panelHandle);  //Dim and highlight the controls which need to be.
			break;
		}
	return 0;
}

/************************okAutoCalc********************************/
/*  This is called when the OK button on the spline option panel  */
/* is clicked on.												  */
/******************************************************************/
int CVICALLBACK okSplineOpPanel (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	double *xPoints=malloc(kMAXNUMOFPOINTS*sizeof(double));	//Allocate memory for the x values to be stored.
	double *yPoints=malloc(kMAXNUMOFPOINTS*sizeof(double));	//Allocate memory for the x values to be stored.
	int index=0; 		//Current element upto in for loop.
	int numOfPoints=0;  //The number of points which were entered.
	
	switch (event)
		{
		case EVENT_COMMIT:
				for (index =0; index<kMAXNUMOFPOINTS; index++)  //Go through all the array elements and get all the allocated numbers.
				{
					if (((xyPoints *)((panelStruct *)callbackData)->otherData)[index].deleted==0)	//If this points hasn't been deleted then...
					{
						xPoints[numOfPoints]=((xyPoints *)((panelStruct *)callbackData)->otherData)[index].channelNum;	//Put the current x value into the xPoints array.
						yPoints[numOfPoints]=((xyPoints *)((panelStruct *)callbackData)->otherData)[index].tThetaValue; //Put the current y value into the yPoints array.
						numOfPoints++;  //Add one to the number of points which haven't been deleted.
					}
				}
				if (calPoints!=NULL) //if the calPoints has been allocated already then..
				{
					free(calPoints); //Free the memory that it is using.
					calPoints= NULL; //Just to make it nice set pointer to NULL.
				}
				calPoints = malloc(sizeof(calPoint)*numOfPoints); //Allocate enough memory for the calPoints.
				for (index=0; index<numOfPoints; index++) //Go through all the points which were entered.
				{
					calPoints[index].xPoint=xPoints[index];  //Put the current x value into calPoints.
					calPoints[index].yPoint=yPoints[index];  //Put the current y value into calPoints.
				}
				header.numOfCalibPoints=numOfPoints; //Set the calibaration value to the number of values entered.
				spline(calPoints, header.numOfCalibPoints, 0, 1);  //Set up the y dashed values for the spline curves.
				cleanUpAfterSplineOpPanel(((xyPoints *)((panelStruct *)callbackData)->otherData)); //Get ride of the memory allocated.
				RemovePopup(((panelStruct*)callbackData)->panelHandle);  //Remove the sline option panel from the screen.
				windowToClose((panelStruct *)callbackData);
			break;
		}
	free(xPoints);  //Release the memory which was allocated for the xPoints array.
	free(yPoints);	//Release the memory which was allocated for the yPoints array.
	return 0;
}


/*****************addPointsToList**********************/
/*	This is to take the current values in calPoints   */
/* and display them in the list box in the spline 	  */
/* option window.									  */
/******************************************************/
void addPointsToList(int panelHandle, xyPoints *thePoints)
{
	int index;			//The index of the point which is being added to the list box.
	int arrayIndex;     //The index of where in the xyPoints array the point was put.
	char listItemLine[100];  //A string for the line which is to be displayed in the list box.
	
	for (index=0; index<header.numOfCalibPoints; index++)  //Go through all the points in the calPoints array.
	{
			sprintf(listItemLine, "%8.4f -> %8.4f", calPoints[index].yPoint, calPoints[index].xPoint);  //Create the line which will be displayed in the list box.						//Create the line which is to be displayed by the list box.
			arrayIndex=addNumsToArray(calPoints[index].yPoint, calPoints[index].xPoint, thePoints);	//Add the values read to the array and get the index of where in the array they where put.
			if (arrayIndex!=-1)	//If there was a space in the array to put the values then...
			{
				InsertListItem(panelHandle, kSPLINEOPT_kDATAPOINTLIST, -1, listItemLine, arrayIndex);  //Add the line to the list box and set it's value to the index of the number in the array.
			}
	}
}

/***************************saveAsDefaultMenu****************/
/*   This is called when the save as default menu item is   */
/* clicked on the from the default menu. It writes all the  */
/* current options into the default option file.            */
/************************************************************/
void CVICALLBACK saveAsDefaultMenu (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	double *xPoints=malloc(kMAXNUMOFPOINTS*sizeof(double));	//Allocate memory for the x values to be stored.
	double *yPoints=malloc(kMAXNUMOFPOINTS*sizeof(double));	//Allocate memory for the x values to be stored.
	int index=0; 		//Current element upto in for loop.
	int numOfPoints=0;  //The number of points which were entered.
	char optionFileName[300];  //This is the path to the default option file.
	
	if (0==ConfirmPopup("Warning", "This will over write any other options in the default file.\nDo you still whant to do this?"))
	{
		return;
	}
	for (index =0; index<kMAXNUMOFPOINTS; index++)  //Go through all the array elements and get all the allocated numbers.
	{
		if (((xyPoints *)((panelStruct *)callbackData)->otherData)[index].deleted==0)	//If this points hasn't been deleted then...
		{
			xPoints[numOfPoints]=((xyPoints *)((panelStruct *)callbackData)->otherData)[index].channelNum;	//Put the current x value into the xPoints array.
			yPoints[numOfPoints]=((xyPoints *)((panelStruct *)callbackData)->otherData)[index].tThetaValue; //Put the current y value into the yPoints array.
			numOfPoints++;  //Add one to the number of points which haven't been deleted.
		}
	}
	if (calPoints!=NULL) //if the calPoints has been allocated already then..
	{
		free(calPoints); //Free the memory that it is using.
		calPoints= NULL; //Just to make it nice set pointer to NULL.
	}
	calPoints = malloc(sizeof(calPoint)*numOfPoints); //Allocate enough memory for the calPoints.
	for (index=0; index<numOfPoints; index++) //Go through all the points which were entered.
	{
		calPoints[index].xPoint=xPoints[index];  //Put the current x value into calPoints.
		calPoints[index].yPoint=yPoints[index];  //Put the current y value into calPoints.
	}
	header.numOfCalibPoints=numOfPoints; //Set the calibaration value to the number of values entered.
	spline(calPoints, header.numOfCalibPoints, 0, 1);  //Set up the y dashed values for the spline curves.
	sprintf(optionFileName, "%s\\Default.pre", projectDir); //Make the path name to the default option file 
	saveOptionsFile(optionFileName, 1); //Save the files to the default file writting over the old ones.
	free(xPoints);		//Free the memory which was allocated for xPoints and yPoints.
	free(yPoints);
}

int CVICALLBACK splineOptionPanel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
				cleanUpAfterSplineOpPanel(((xyPoints *)((panelStruct *)callbackData)->otherData)); //Free the allocated memory for the xyPoint Array
				RemovePopup(((panelStruct*)callbackData)->panelHandle);	//Remove the panel from the screen.
				windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}

void CVICALLBACK importXfit (int menuBar, int menuItem, void *callbackData,
		int panel)
{
	panelStruct *panelInfo = NULL;
	
	panelInfo = createPanelStruct(LoadPanel(panel, "DiffConv.uir", kLOADCALIB), panel, malloc(sizeof(int)));	//create the panel structure for the Standards panel and load it in.
	initStandard(panelInfo, ((panelStruct*)callbackData));
	InstallPopup(panelInfo->panelHandle);				//Display the current options panel as a Popup.
}

