/**********************************************************************/
/*   By Stefan Pantos. E-Mail: Stefuser@yahoo.com                     */
/*	 Last Modified: 21/07/2000										  */
/**********************************************************************/
/******************************************************************************
*
*    Copyright (C) 2000 Stefan Pantos
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
***********************************************************************/
#include <utility.h>
#include <userint.h>
#include "DiffConv.h"
#include "Standards.h"
#include "SplineOptionPanel.h"

double *standard = NULL;
int numOfStandards = 0;
double *observed = NULL;
int numOfObserved = 0;
panelStruct *parentPanel;

void cleanUpStandardsWindow(panelStruct *panelInfo);

void dimStandards(panelStruct *panelInfo)
{
	int numOfItems;
	
	GetNumListItems(panelInfo->panelHandle, kLOADCALIB_kSTANDARDS, &numOfItems);
	if (numOfItems <= 0)
	{
		SetCtrlAttribute(panelInfo->panelHandle, kLOADCALIB_kOBSERVED, ATTR_DIMMED, 1);	
	}
	else
	{
		SetCtrlAttribute(panelInfo->panelHandle, kLOADCALIB_kOBSERVED, ATTR_DIMMED, 0);	
	}
	GetNumListItems(panelInfo->panelHandle, kLOADCALIB_kPOINTS, &numOfItems); 
	if (numOfItems <= 0)
	{
		SetCtrlAttribute(panelInfo->panelHandle, kLOADCALIB_kOKBUTTON, ATTR_DIMMED, 1);	
	}
	else
	{
		SetCtrlAttribute(panelInfo->panelHandle, kLOADCALIB_kOKBUTTON, ATTR_DIMMED, 0);	
	}
}

int nextWord(char line[], char word[], char delimeter, int startPos, int maxPos)
{
	int i = startPos;
	int wordPos = 0;
	int space = 0;
	
	if (delimeter == line[i])
	{
		space = 1;
	}
	while ((space == 1 || line[i] != delimeter) && maxPos > i && line[i] != 0x00)
	{
		if (space == 1 && line[i]!=delimeter)
		{
			space = 0;
		}
		if (space == 0)
		{
			word[wordPos] = line[i];
			wordPos ++;
		}
		i++;
	}
	word[wordPos] = 0x00;
	return i;
}

int findXFitType(int fileHandle)
{
	long currentFilePos;
	char line[400];
	char word[400];
	int result;
	int position;
	int wordLength;
	int i;
	
	SetFilePtr(fileHandle, 0x0, 0);
	result = readLine(fileHandle, line, 400);
	
	if (strlen(line) < 33)
	{
		position = nextWord(line, word, ' ',  0, 400);
		wordLength = strlen(word);
		i = 0;
		while ((word[i] >= '0' && word[i] <= '9') || word[i] == '.' ||word[i] == ' ' || word[i] == '\t')
		{
			i++;
		}
		if (!((word[i] >= '0' && word[i] <= '9') || word[i] == ' ' || word[i] == '.' || word[i] == '\t' || word[i] == 0x00))
		{
			return -1;
		}
		SetFilePtr(fileHandle, 0x0, 0);
		return 0;
	}
	else
	{
		position = nextWord(line, word, ' ',  25, 400);	
		if (strCmp(word, "Th2") == 0)
		{
			return 23; 
		}
		else
		{
			position = nextWord(line, word, ' ',  48, 400);
			if (strCmp(word, "Th2") == 0)
			{
				return 42; 
			}
		}
	}
	return -1;
}

int getStandardPoints(int fileHandle, double **standardData)
{
	char line[400];
	char word[400];
	int dataStart = 0, numOfPoints = 0, i;
	int bytesRead;
	
	SetFilePtr(fileHandle, 0x00, 0);
	do
	{
		bytesRead  = readLine(fileHandle, line, 400);
		nextWord(line, word, ' ', 0, 400);
	}
	while (strCmp(word, "DATA") && bytesRead >= 0);
	dataStart = SetFilePtr(fileHandle, 0, 1); 
	if (bytesRead <= 0)
	{
		numOfStandards = 0;
		MessagePopup("No Points", "The selected standard has no points.");
		return 0;
	}
	while (readLine(fileHandle, line, 400) >0)
	{
		numOfPoints ++;
	}
	if (standardData != NULL)
	{
		free(*standardData);
		numOfStandards = 0;
		*standardData = NULL;
	}
	numOfStandards = numOfPoints;
	*standardData = malloc(sizeof(double) * numOfPoints);
	SetFilePtr(fileHandle, dataStart, 0);
	numOfPoints = 0;
	while (readLine(fileHandle, line, 400) >0)
	{
		nextWord(line, word, ' ', 0, 400);
		(*standardData)[numOfPoints] = atof(word);
		numOfPoints ++;
	}
	return 1;
}

int readXFitFile(int fileHandle, double **observedData, int tThetaPosition)
{
	int lineNum = 0;
	char line[400];
	char number[400];
	
	SetFilePtr(fileHandle, 0x0, 0);
	while (readLine(fileHandle, line, 400)>0)
	{
		lineNum ++;
	}
	SetFilePtr(fileHandle, 0x0, 0);
	if (tThetaPosition != 0)
	{
		lineNum --;
		readLine(fileHandle, line, 400);
	}
	if (*observedData != NULL)
	{
		free(*observedData);
		numOfObserved = 0; 
	}
	if (lineNum <= 0)
	{
		MessagePopup("No Points", "There were no points in the XFit file.");
		return 0;
	}
	*observedData = malloc(sizeof(double) * lineNum);
	numOfObserved = lineNum;
	lineNum = 0;
	while (readLine(fileHandle, line, 400)>0)
	{
		nextWord(line, number, ' ', tThetaPosition, strlen(line));
		(*observedData)[lineNum] = atof(number);
		lineNum++;
	}
	return 1;
}

int getNameOfStd(char line[], panelStruct *panelInfo, char name[200])
{
	int posInLine = 0;
	char word[100];
	
	posInLine =  nextWord(line, word, ' ', posInLine, 400);
	if (strCmp("name", word) == 0 || strCmp("name=", word) == 0)
	{
		posInLine =  nextWord(line, word, '\'', posInLine, 400); 
		posInLine =  nextWord(line, word, '\'', posInLine, 400); 
		strcpy(name, word);
		return 1;
	}
	return 0;
}

void AddFileToList(panelStruct *panelInfo, char *fileName)
{
	char *filePath = malloc(300);
	char line[400];
	char name[200];
	int inputFile;
	int result= 0;
	
	sprintf(filePath, "%s\\Standards\\%s", projectDir, fileName);
	if ((inputFile=OpenFile(filePath, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))<0)
	{
		return;
	}
	do
	{
		result = readLine(inputFile, line, 400);
		if (result >= 0)
		{
			result = getNameOfStd(line, panelInfo, name);
			if (result == 1)
			{
				break;
			}
		}
	}while (result >= 0);
	if (result == 1)
	{
		InsertListItem(panelInfo->panelHandle, kLOADCALIB_kSTANDARDS, 0, name, filePath);
	}
	free(filePath);
	CloseFile(inputFile);
}

void checkForStandards(panelStruct *panelInfo)
{
	int result;
	char fileName[300];
	char searchPath[300];
	
	sprintf(searchPath, "%s%s", projectDir, "\\Standards\\*.std");
	result = GetFirstFile(searchPath, 1, 0, 0, 0, 0, 0, fileName);
	while (result == 0)
	{
		AddFileToList(panelInfo, fileName);	
		result = GetNextFile(fileName);
	}
}

void initStandard(panelStruct *panelInfo, panelStruct *parentPanelInfo)
{		
	parentPanel = parentPanelInfo;
	SetCtrlAttribute(panelInfo -> panelHandle, kLOADCALIB_kOKBUTTON, ATTR_CALLBACK_DATA, panelInfo);
	SetCtrlAttribute(panelInfo -> panelHandle, kLOADCALIB_kCANCELBUTTON, ATTR_CALLBACK_DATA, panelInfo);
	SetCtrlAttribute(panelInfo -> panelHandle, kLOADCALIB_kOBSERVED, ATTR_CALLBACK_DATA, panelInfo);
	SetPanelAttribute(panelInfo -> panelHandle, ATTR_CALLBACK_DATA, panelInfo); 
	checkForStandards(panelInfo);
	dimStandards(panelInfo);
}

int CVICALLBACK okButtonStandards (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int arrayIndex, numOfPoints, i;
	panelStruct *callback;
	char line[100];
	
	switch (event)
		{
		case EVENT_COMMIT:
				GetCtrlAttribute(((panelStruct *)callbackData)->parentPanelHandle, kSPLINEOPT_kREMOVEALL, ATTR_CALLBACK_DATA, ((void **)&callback));
				ClearListCtrl(((panelStruct *)callbackData)->parentPanelHandle, 2);
				for (i = 0; i<kMAXNUMOFPOINTS; i++)
				{
					((xyPoints *)(((panelStruct *)callback)->otherData))[i].deleted = 0x01;	
				}
				numOfPoints = numOfStandards;
				if (numOfPoints > numOfObserved)
				{
					numOfPoints = numOfObserved;	
				}
				for (i = 0; i<numOfPoints && i < kMAXNUMOFPOINTS; i++)
				{
					arrayIndex = addNumsToArray(standard[i], observed[i], ((xyPoints *)(((panelStruct *)callback)->otherData)));
					sprintf(line, "%8.04f  -> %8.04f", standard[i], observed[i]);
					InsertListItem(((panelStruct *)callbackData)->parentPanelHandle, 2, -1, line, arrayIndex);  
				}
				RemovePopup(((panelStruct*)callbackData)->panelHandle);	//Remove the panel from the screen.
				InstallPopup(((panelStruct *) callbackData)->parentPanelHandle);
				cleanUpStandardsWindow((panelStruct *) callbackData);
				windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}

int CVICALLBACK cancelButtonStandards (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
				RemovePopup(((panelStruct*)callbackData)->panelHandle);	//Remove the panel from the screen.
				InstallPopup(((panelStruct *) callbackData)->parentPanelHandle);
				cleanUpStandardsWindow((panelStruct *) callbackData);			  
				windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}

void updatePointList(int panelHandle)
{
	int numOfPoints =0;
	int i;
	char listItem[100];
	
	ClearListCtrl(panelHandle, kLOADCALIB_kPOINTS);
	numOfPoints = numOfStandards;
	if (numOfObserved < numOfPoints)
	{
		numOfPoints = numOfObserved;
	}
	for (i = 0; i < numOfPoints; i++)
	{
		sprintf(listItem, "%8.04f -> %8.04f", standard[i], observed[i]);
		InsertListItem(panelHandle, kLOADCALIB_kPOINTS, -1, listItem, 0); 
	}
}

int CVICALLBACK openXfitPeaks (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char standardFilePath[300];
	char observedFilePath[300];
	int inputFile, standeredFile;
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(((panelStruct *) callbackData)->panelHandle, kLOADCALIB_kSTANDARDS, standardFilePath);
			if (!FileSelectPopup(projectDir, "*.txt", "*.txt", "Select Option File", VAL_OK_BUTTON, 0, 1, 1, 0, observedFilePath))	//Display the File Select Window for the user to select the name of the file.
			{
				return 0;	
			}
			if ((inputFile = OpenFile(observedFilePath, VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))< 0)  //open xfitFile file
			{
				MessagePopup("File Error", "Couldn't Open Xfit File. You may have it open or it my not exist.");
				return 0;
			}
			if ((standeredFile  = OpenFile(standardFilePath , VAL_READ_ONLY, VAL_APPEND, VAL_ASCII))< 0)
			{
				MessagePopup("File Error", "Couldn't Open standered File. You may have it open or it my not exist.");
				return 0;	
			}
			if (!getStandardPoints(standeredFile, &standard))
			{
				CloseFile(inputFile);
				CloseFile(standeredFile);
				dimStandards(((panelStruct *) callbackData));
				return 0;
			}
			if (!readXFitFile(inputFile, &observed, findXFitType(inputFile)))
			{
				CloseFile(inputFile);
				CloseFile(standeredFile);
				dimStandards(((panelStruct *) callbackData));
				return 0;
			}
			CloseFile(inputFile);
			CloseFile(standeredFile);
			updatePointList(panel);
			dimStandards(((panelStruct *) callbackData));
			break;
		}
	return 0;
}

int CVICALLBACK standardMainMethod (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:
			dimStandards(((panelStruct *) callbackData));
			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
				RemovePopup(((panelStruct*)callbackData)->panelHandle);	//Remove the panel from the screen.
				InstallPopup(((panelStruct *) callbackData)->parentPanelHandle);
				cleanUpStandardsWindow((panelStruct *) callbackData);			  
				windowToClose((panelStruct *) callbackData);
			break;
		}
	return 0;
}

void cleanUpStandardsWindow(panelStruct *panelInfo)
{
	if (observed != NULL)
	{
		free(observed);
		numOfObserved = 0; 
		observed = NULL;
	}
	if (standard != NULL)
	{
		free(standard);
		numOfStandards = 0;  
		standard = NULL;
	}
}
